/*
 * detumbling_controller.c
 *
 * Created: 24.03.2014 13:32:10
 *  Author: yvind Rein
 *  Edited By: Marius Westgaard
 */ 

#include <stdint.h>

#define COIL_MAX 32767.0F			// Maximum coil value
#define MAGNETOMETER_SCALING 1.0F/230.0F			// Found in datasheet

volatile float B_B_dot[3];
volatile float B_B_dot_prev[3];
volatile float B_B_prev[3];
volatile float m_B[3];
volatile float B_B_float[3];

// B-dot estimator constants
#define A 0.5
#define B 0.7

// Controller constants
#define D 0.0001					// Gain found by simulations
#define OUTPUT_SCALING 121001.0F	// int16_t_max/m_B_max = OUTPUT_SCALING

int detumbling_controller_intialize(){
	for(int i = 0; i < 3; i++){
		B_B_dot_prev[i] = 0;
		B_B_dot[i] = 0;
		B_B_prev[i] = 0;
		m_B[i] = 0;
		B_B_float[i] = 0;
	}
	return 1;
}

int B_dot_estimate(float B_B[3]){
	for(int i = 0; i < 3; i++){
		B_B_dot_prev[i] = B_B_dot[i];
		B_B_dot[i] = ((A*B_B_dot_prev[i]) + (B* (B_B[i] - B_B_prev[i])));
		B_B_prev[i] = B_B[i];
	}
	return 1;
}

int detumbling_controller_get_output(int16_t B_B[3], int16_t coil_output[3]){
	
	B_B_float[0] = MAGNETOMETER_SCALING*B_B[0];
	B_B_float[1] = MAGNETOMETER_SCALING*B_B[1];
	B_B_float[2] = MAGNETOMETER_SCALING*B_B[2];
	
	B_dot_estimate(&B_B_float);
	
	m_B[0] = -D*B_B_dot[0];
	m_B[1] = -D*B_B_dot[1];
	m_B[2] = -D*B_B_dot[2];
	
	m_B[0] = m_B[0]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	m_B[1] = m_B[1]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	m_B[2] = m_B[2]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	
	// Make sure there are no overflow for the x gain
	if (m_B[0] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[0] = COIL_MAX;
	}
	else if (m_B[0] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[0] = -COIL_MAX;
	}
	else {
		coil_output[0] = OUTPUT_SCALING*m_B[0];
	}
	
	// Make sure there are no overflow for the y gain
	if (m_B[1] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[1] = COIL_MAX;
	}
	else if (m_B[1] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[1] = -COIL_MAX;
	}
	else {
		coil_output[1] = OUTPUT_SCALING*m_B[1];
	}
	
	// Make sure there are no overflow for the z gain
	if (m_B[2] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[2] = COIL_MAX;
	}
	else if (m_B[2] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[2] = -COIL_MAX;
	}
	else {
		coil_output[2] = OUTPUT_SCALING*m_B[2];
	}
	
	return 1;
}
