/*
 * pointing_controller.c
 *
 * Created: 15.10.2014 19:17:22
 *  Author: Marius Westgaard
 */ 


#include <stdint.h>

#define COIL_MAX 32767.0F					// Maximum coil value
#define MAGNETOMETER_SCALING 1.0/230.0		// Found in datasheet

volatile float m_B[3];
volatile float w_B_OB[3];
volatile float q[4];
volatile float B_B_float[3];

// Controller constants
#define P 0.0000018					// Gain found by simulations
#define D 0.0014					// Gain found by simulations
#define OUTPUT_SCALING 121001.0F	// int16_t_max/m_B_max = OUTPUT_SCALING

int pointing_controller_intialize(){
		for(int i = 0; i < 3; i++){
			m_B[i] = 0;
			w_B_OB[i] = 0;
			q[i] = 0;
			B_B_float[i] = 0;
		}
		q[3] = 0;
		return 1;
}

int pointing_controller_get_output(int16_t B_B[3], float estimator_data[7], int16_t coil_output[3]){
	B_B_float[0] = MAGNETOMETER_SCALING*B_B[0];
	B_B_float[1] = MAGNETOMETER_SCALING*B_B[1];
	B_B_float[2] = MAGNETOMETER_SCALING*B_B[2];
	
	q[0] = estimator_data[0];
	q[1] = estimator_data[1];
	q[2] = estimator_data[2];
	q[3] = estimator_data[3];
	
	w_B_OB[0] = estimator_data[4];
	w_B_OB[1] = estimator_data[5];
	w_B_OB[2] = estimator_data[6];
		
	m_B[0] = (-P*(B_B_float[1]*q[3] - B_B_float[2]*q[2]) -D*(B_B_float[1]*w_B_OB[2] - B_B_float[2]*w_B_OB[1]));
	m_B[1] = (-P*(B_B_float[2]*q[1] - B_B_float[0]*q[3]) -D*(B_B_float[2]*w_B_OB[0] - B_B_float[0]*w_B_OB[2]));
	m_B[2] = (-P*(B_B_float[0]*q[2] - B_B_float[1]*q[1]) -D*(B_B_float[0]*w_B_OB[1] - B_B_float[1]*w_B_OB[0]));

	m_B[0] = m_B[0]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	m_B[1] = m_B[1]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	m_B[2] = m_B[2]/(B_B_float[0]*B_B_float[0] + B_B_float[1]*B_B_float[1] + B_B_float[2]*B_B_float[2]);
	
	// Make sure there are no overflow for the x gain
	if (m_B[0] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[0] = COIL_MAX;
	} 
	else if (m_B[0] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[0] = -COIL_MAX;
	}
	else {
		coil_output[0] = OUTPUT_SCALING*m_B[0];
	}
	
	// Make sure there are no overflow for the y gain
	if (m_B[1] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[1] = COIL_MAX;
	}
	else if (m_B[1] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[1] = -COIL_MAX;
	}
	else {
		coil_output[1] = OUTPUT_SCALING*m_B[1];
	}
	
	// Make sure there are no overflow for the z gain
	if (m_B[2] > (COIL_MAX/OUTPUT_SCALING)) {
		coil_output[2] = COIL_MAX;
	}
	else if (m_B[2] < -(COIL_MAX/OUTPUT_SCALING)) {
		coil_output[2] = -COIL_MAX;
	}
	else {
		coil_output[2] = OUTPUT_SCALING*m_B[2];
	}
	
	return 1;
}
