/*
 * frameTrans.h
 *
 *  Created: Feb 16, 2015 10:14:22 AM
 *  Author: Antoine Pignede
 *
 *  This file declares the Frame Transformations
 */

#ifndef FRAMETRANS_H_
#define FRAMETRANS_H_

	typedef struct{
		double l;
		double l1;
		double f;
		double d;
		double omega;
	} Delaunay;

/* Declare the transformation functions used directly in the algorithms */
	void frameEcef2geod(double recef[3], double rgeod[3]);
	void frameGeod2ecef(double rgeod[3], double recef[3]);
	void frameEcef2eci(double recef[3],Time timeUTC, double reci[3]);
	void frameEci2ecef(double reci[3],Time timeUTC, double recef[3]);
	void frameEci2teme(double reci[3],Time timeUTC, double rteme[3]);
	void frameTeme2eci(double rteme[3],Time timeUTC, double reci[3]);

/* Declare the help functions */
	// for frameEcef2geod
	void frameLatitudeRecur(double lat_in, double z, double rd, int8_t iter, double *latitude, double *Nphi);

	// for eci, ecef, teme transformation matrices
	Delaunay frameFundarg(double ttt);
	void frameNutation(double ttt, double *deltapsi, double *meaneps, double *omega, double nut[9]);
	void framePolarm(double jdutc, double pm[9]);
	void framePrecess(double ttt, double prec[9]);
	void frameSidereal(double jdutc,double deltapsi,double meaneps, double omega, double st[9]);
	void frameTruemean(double ttt, double nutteme[9]);

	// transformation matrix for geomagnetic field vector in ned to ecef
	void frameRotationNed2ecef(double sinlat, double coslat, double sinlon, double coslon, double Ned2ecef[9]);

#endif /* FRAMETRANS_H_ */
