/*
 * orbitPred.h
 *
 *  Created: Feb 17, 2015 9:10:27 AM
 *  Author: Antoine Pignede
 *
 *  This file declares the Orbit Position And Velocity Prediction
 */

#ifndef ORBITPRED_H_
#define ORBITPRED_H_

	/*
	 * Declare the struct with the parameters for tracking satellites in SGP4 algorithm
	 * The first satrec struct element is an error message. If everything is
	 *   fine it will be empty, otherwise a short description of the error is
	 *   given. In orbitTwoline2rv possible errors in the tle string are
	 *   detected. In orbitSgp4 values outside the expected range are
	 *   detected and zero position and velocity returned. The only
	 *   error to really stop a function is if the TLE input string in
	 *   orbitTwoline2rv is less than 138 characters long.
	 */
	typedef struct{
		char error[128];
		// the translated two-line element set, see orbitTwoline2rv
		long int satnum;
		char classification;
		char intldesg[16];
		int epochyr;
		double epochdays;
		double ndot;
		double nddot;
		double bstar;
		int ephemtype;
		long int elnum;
		double inclo;
		double nodeo;
		double ecco;
		double argpo;
		double mo;
		double no;
		long int revnum;
		// parameter added by orbitTwoline2rv
		double jdsatepoch;
		// parameters added by orbitSgp4init
		int isimp; // For perigee < 220 km, set isimp=1, truncate some equations and drop some terms
		double argpdot;
		double aycof;
		double con41;
		double cc1;
		double cc4;
		double cc5;
		double d2;
		double d3;
		double d4;
		double delmo;
		double eta;
		double mdot;
		double nodecf;
		double nodedot;
		double omgcof;
		double sinmao;
		double t;
		double t2cof;
		double t3cof;
		double t4cof;
		double t5cof;
		double x1mth2;
		double x7thm1;
		double xlcof;
		double xmcof;
	} Satrec;

	// Declare functions for TLE processing and SGP4 algorithm (deep space removed!)
	Satrec orbitTwoline2rv(char tle[139]);
	void orbitSgp4init(Satrec *satrec);
	void orbitSgp4(Satrec *satrec, Time timeUTC, double reci[3], double veci[3]);

#endif /* ORBITPRED_H_ */
