/*
 * testC.c
 *
 *  Created: Feb 11, 2015 1:58:33 PM
 *  Author: Antoine Pignede
 *
 *  Use this file to test the C functions. Each function is used at
 *    least once with known input and output values given in the sources.
 *  "Sol" behind the variable name means that this is the value given in
 *    the Solution and that the Algorithm should find.
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "constants.h"
#include "timeConv.h"
#include "frameTrans.h"
#include "sunPred.h"
#include "orbitPred.h"
#include "magPred.h"

void verification(){
	int testCount = 0;
	int errorCount = 0;
	double absTol = 0.1; // tolerance for the absolute difference when comparing matrices
	double relTol = 0.5; // tolerance for the relative difference when comparing matrices
	clock_t start; // to measure the time taken for the IGRF...
	clock_t end; // ...and WMM functions and find out which...
	int cpu_time_used; // ...one is fastest

/* Test the Time Conversions */
	// Excerpt of ex3_1415.m from Vallado (2013) Fundamentals of Astrodynamics and Applications
	Time timeSol;
	Time timeTTSol;
	Time time;
	Time timeTT;
	timeSol.year=2004;
	timeSol.mon = 4;
	timeSol.day = 6;
	timeSol.hr =  7;
	timeSol.min= 51;
	timeSol.sec= 28.386009;
	timeTTSol.year=2004;
	timeTTSol.mon = 4;
	timeTTSol.day = 6;
	timeTTSol.hr =  7;
	timeTTSol.min= 52;
	timeTTSol.sec= 32.570009;
	double jd = timeDatetime2jd(timeSol); // TEST!
	testCount = testCount + 1;
	double jdSol = 2453101.8274118751;
	if (fabs(jd - jdSol) > (1.0/86400)){ // If difference greater than 1s
		errorCount = errorCount +1;
		printf("timeDatetime2jd error! jd:%18.10f not jdSol:%18.10f\n",jd,jdSol);
	}
	else printf("Test %d timeDatetime2jd OK!\n",testCount);

	time = timeJd2datetime(jdSol); // TEST!
	testCount = testCount + 1;
	if ((time.year != timeSol.year) || (time.mon != timeSol.mon) || (time.day != timeSol.day) || (time.hr != timeSol.hr) || (time.min != timeSol.min) || (fabs(time.sec - timeSol.sec) > 1.0)){
		errorCount = errorCount +1;
		printf("timeJd2datetime error! time not timeSol\n");
		printf("time:   %d/%d/%d %d:%d:%18.10f\n",time.year,time.mon,time.day,time.hr,time.min,time.sec);
		printf("timeSol:%d/%d/%d %d:%d:%18.10f\n",timeSol.year,timeSol.mon,timeSol.day,timeSol.hr,timeSol.min,timeSol.sec);
	}
	else printf("Test %d timeJd2datetime OK!\n",testCount);

	double days = timeDatetime2days(timeSol); // TEST!
	testCount = testCount + 1;
	double daysSol = 97.3274118751;
	if (fabs(days - daysSol) > (1.0/86400)){ // If difference greater than 1s
		errorCount = errorCount +1;
		printf("timeDatetime2days error! days:%18.10f not daysSol:%18.10f\n",days,daysSol);
	}
	else printf("Test %d timeDatetime2days OK!\n",testCount);

	time = timeDays2datetime(timeSol.year,daysSol); // TEST
	testCount = testCount + 1;
	if ((time.year != timeSol.year) || (time.mon != timeSol.mon) || (time.day != timeSol.day) || (time.hr != timeSol.hr) || (time.min != timeSol.min) || (fabs(time.sec - timeSol.sec) > 1.0)){
		errorCount = errorCount +1;
		printf("timeDays2datetime error! time not timeSol\n");
		printf("time:   %d/%d/%d %d:%d:%18.10f\n",time.year,time.mon,time.day,time.hr,time.min,time.sec);
		printf("timeSol:%d/%d/%d %d:%d:%18.10f\n",timeSol.year,timeSol.mon,timeSol.day,timeSol.hr,timeSol.min,timeSol.sec);
	}
	else printf("Test %d timeDays2datetime OK!\n",testCount);

	double years = timeDatetime2years(timeSol); // TEST!
	testCount = testCount + 1;
	double yearsSol =  2004.2631896499;
	if (fabs(years - yearsSol) > (1.0/86400)){ // If difference greater than 1s
		errorCount = errorCount +1;
		printf("timeDatetime2years error! years:%18.10f not yearsSol:%18.10f\n",years,yearsSol);
	}
	else printf("Test %d timeDatetime2years OK!\n",testCount);

	time = timeYears2datetime(yearsSol); // TEST
	testCount = testCount + 1;
	if ((time.year != timeSol.year) || (time.mon != timeSol.mon) || (time.day != timeSol.day) || (time.hr != timeSol.hr) || (time.min != timeSol.min) || (fabs(time.sec - timeSol.sec) > 1.0)){
		errorCount = errorCount +1;
		printf("timeYears2datetime error! time not timeSol\n");
		printf("time:   %d/%d/%d %d:%d:%18.10f\n",time.year,time.mon,time.day,time.hr,time.min,time.sec);
		printf("timeSol:%d/%d/%d %d:%d:%18.10f\n",timeSol.year,timeSol.mon,timeSol.day,timeSol.hr,timeSol.min,timeSol.sec);
	}
	else printf("Test %d timeYears2datetime OK!\n",testCount);

	double jc = timeJd2jc(jdSol); // TEST
	testCount = testCount + 1;
	double jcSol = 0.0426236116;
	if (fabs(jc - jcSol) > (1.0/3.1558e+09)){ // If difference greater than 1s
		errorCount = errorCount +1;
		printf("timeJd2jc error! jc:%18.10f not jcSol:%18.10f\n",jc,jcSol);
	}
	else printf("Test %d timeJd2jc OK!\n",testCount);

	jd = timeJc2jd(jc); // TEST
	testCount = testCount + 1;
	if (fabs(jd - jdSol) > (1.0/86400)){ // If difference greater than 1s
		errorCount = errorCount +1;
		printf("timeJc2jd error! jd:%18.10f not jdSol:%18.10f\n",jd,jdSol);
	}
	else printf("Test %d timeJc2jd OK!\n",testCount);

	timeTT = timeUtc2tt(timeSol); // TEST
	testCount = testCount + 1;
	if ((timeTT.year != timeTTSol.year) || (timeTT.mon != timeTTSol.mon) || (timeTT.day != timeTTSol.day) || (timeTT.hr != timeTTSol.hr) || (timeTT.min != timeTTSol.min) || (fabs(timeTT.sec - timeTTSol.sec) > 1.0)){
		errorCount = errorCount +1;
		printf("timeUtc2tt error! timeTT not timeTTSol\n");
		printf("timeTT:   %d/%d/%d %d:%d:%18.10f\n",timeTT.year,timeTT.mon,timeTT.day,timeTT.hr,timeTT.min,timeTT.sec);
		printf("timeTTSol:%d/%d/%d %d:%d:%18.10f\n",timeTTSol.year,timeTTSol.mon,timeTTSol.day,timeTTSol.hr,timeTTSol.min,timeTTSol.sec);
	}
	else printf("Test %d timeUtc2tt OK!\n",testCount);

	time = timeTt2utc(timeTTSol); // TEST
	testCount = testCount + 1;
	if ((time.year != timeSol.year) || (time.mon != timeSol.mon) || (time.day != timeSol.day) || (time.hr != timeSol.hr) || (time.min != timeSol.min) || (fabs(time.sec - timeSol.sec) > 1.0)){
		errorCount = errorCount +1;
		printf("timeTt2utc error! time not timeSol\n");
		printf("time:   %d/%d/%d %d:%d:%18.10f\n",time.year,time.mon,time.day,time.hr,time.min,time.sec);
		printf("timeSol:%d/%d/%d %d:%d:%18.10f\n",timeSol.year,timeSol.mon,timeSol.day,timeSol.hr,timeSol.min,timeSol.sec);
	}
	else printf("Test %d timeTt2utc OK!\n",testCount);

	double gst = timeGstime(jdSol); // TEST
	testCount = testCount + 1;
	double gstSol = 5.4594896669;
	if (fabs(gst - gstSol) > absTol){
		errorCount = errorCount +1;
		printf("timeGstime error! gst:%18.10f not gstSol:%18.10f\n\n",gst,gstSol);
	}
	else printf("Test %d timeGstime OK!\n\n",testCount);

/* Test the Frame Transformations */
	double tttSol = 0.042623631889;
	Delaunay delaunay = frameFundarg(tttSol); // TEST
	testCount = testCount + 1;
	double lSol = 5.4962562692;
	double l1Sol = 1.6046448385;
	double fSol = 2.9512994807;
	double dSol = 3.4339639764;
	double omegaSol = 0.7435913620;
	if ((fabs(delaunay.l - lSol) > absTol) || (fabs(delaunay.l1 - l1Sol) > absTol) || (fabs(delaunay.f - fSol) > absTol) || (fabs(delaunay.d - dSol) > absTol) || (fabs(delaunay.omega - omegaSol) > absTol)){
		errorCount = errorCount +1;
		printf("frameFundarg error! delaunay not delaunaySol\n");
		printf("l    :%18.10f, lSol    :%18.10f\n",delaunay.l,lSol);
		printf("l1   :%18.10f, l1Sol   :%18.10f\n",delaunay.l1,l1Sol);
		printf("f    :%18.10f, fSol    :%18.10f\n",delaunay.f,fSol);
		printf("d    :%18.10f, dSol    :%18.10f\n",delaunay.d,dSol);
		printf("omega:%18.10f, omegaSol:%18.10f\n",delaunay.omega,omegaSol);
	}
	else printf("Test %d frameFundarg OK!\n",testCount);

	// Excerpt of ex3_1415.m from Vallado (2013) Fundamentals of Astrodynamics and Applications
    double prec[9];
    framePrecess(tttSol, prec); // TEST
    testCount = testCount + 1;
    double precSol[9] = {0.99999945998, -0.00095314992, -0.00041417739, 0.00095314992, 0.99999954575, -0.00000019739, 0.00041417739, -0.00000019739, 0.99999991423};
    double absError[9];
    double relError[9];
    int same = 1;
    int i;
    for (i=0; i<=8; i++){
    	absError[i] = prec[i]-precSol[i];
    	relError[i] = absError[i] / prec[i];
    	if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
    	if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
    }
	if (same == 0){
		errorCount = errorCount +1;
		printf("framePrecess error! prec not precSol\n");
		printf("prec:\n%18.10f %18.10f %18.10f\n",prec[0],prec[3],prec[6]);
		printf("%18.10f %18.10f %18.10f\n",prec[1],prec[4],prec[7]);
		printf("%18.10f %18.10f %18.10f\n",prec[2],prec[5],prec[8]);
		printf("precSol:\n%18.10f %18.10f %18.10f\n",precSol[0],precSol[3],precSol[6]);
		printf("%18.10f %18.10f %18.10f\n",precSol[1],precSol[4],precSol[7]);
		printf("%18.10f %18.10f %18.10f\n",precSol[2],precSol[5],precSol[8]);
	}
	else printf("Test %d framePrecess OK!\n",testCount);

	double deltapsi;
	double meaneps;
	double omega;
	double nut[9];
	frameNutation(tttSol, &deltapsi, &meaneps, &omega, nut); // TEST
	testCount = testCount + 1;
	double nutSol[9] = {0.99999999823, 0.00005461818, 0.00002367925, -0.00005461734, 0.99999999788, -0.00003545941, -0.00002368119, 0.00003545812, 0.99999999909};
	omegaSol = 0.7435907904;
	double meanepsSol = 0.4090831301;
	double deltapsiSol = -0.0000597833;
	same = 1;
	for (i=0; i<=8; i++){
		absError[i] = nut[i]-nutSol[i];
		relError[i] = absError[i] / nut[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameNutation error! nut not nutSol\n");
		printf("nut:\n%18.10f %18.10f %18.10f\n",nut[0],nut[3],nut[6]);
		printf("%18.10f %18.10f %18.10f\n",nut[1],nut[4],nut[7]);
		printf("%18.10f %18.10f %18.10f\n",nut[2],nut[5],nut[8]);
		printf("nutSol:\n%18.10f %18.10f %18.10f\n",nutSol[0],nutSol[3],nutSol[6]);
		printf("%18.10f %18.10f %18.10f\n",nutSol[1],nutSol[4],nutSol[7]);
		printf("%18.10f %18.10f %18.10f\n",nutSol[2],nutSol[5],nutSol[8]);
		printf("omega:   %18.10f, omegaSol:   %18.10f\n",omega,omegaSol);
		printf("meaneps: %18.10f, meanepsSol: %18.10f\n",meaneps,meanepsSol);
		printf("deltapsi:%18.10f, deltapsiSol:%18.10f\n",deltapsi,deltapsiSol);
	}
	else printf("Test %d frameNutation OK!\n",testCount);
	if ((fabs(omega - omegaSol) > absTol) || (fabs(meaneps - meanepsSol) > absTol) || (fabs(deltapsi - deltapsiSol) > absTol)){
		printf("frameNutation warning!\n");
		printf("nut:\n%18.10f %18.10f %18.10f\n",nut[0],nut[3],nut[6]);
		printf("%18.10f %18.10f %18.10f\n",nut[1],nut[4],nut[7]);
		printf("%18.10f %18.10f %18.10f\n",nut[2],nut[5],nut[8]);
		printf("nutSol:\n%18.10f %18.10f %18.10f\n",nutSol[0],nutSol[3],nutSol[6]);
		printf("%18.10f %18.10f %18.10f\n",nutSol[1],nutSol[4],nutSol[7]);
		printf("%18.10f %18.10f %18.10f\n",nutSol[2],nutSol[5],nutSol[8]);
		printf("omega:   %18.10f, omegaSol:   %18.10f\n",omega,omegaSol);
		printf("meaneps: %18.10f, meanepsSol: %18.10f\n",meaneps,meanepsSol);
		printf("deltapsi:%18.10f, deltapsiSol:%18.10f\n",deltapsi,deltapsiSol);
	}

	double st[9];
	frameSidereal(jdSol, deltapsiSol, meanepsSol, omegaSol, st); // TEST
	testCount = testCount + 1;
	double stSol[9] = {0.67952793705, -0.73364963216, 0.00000000000, 0.73364963216, 0.67952793705, 0.00000000000, 0.00000000000, 0.00000000000, 1.00000000000};
	same = 1;
	for (i=0; i<=8; i++){
		absError[i] = st[i]-stSol[i];
		relError[i] = absError[i] / st[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameSidereal error! st not stSol\n");
		printf("st:\n%18.10f %18.10f %18.10f\n",st[0],st[3],st[6]);
		printf("%18.10f %18.10f %18.10f\n",st[1],st[4],st[7]);
		printf("%18.10f %18.10f %18.10f\n",st[2],st[5],st[8]);
		printf("stSol:\n%18.10f %18.10f %18.10f\n",stSol[0],stSol[3],stSol[6]);
		printf("%18.10f %18.10f %18.10f\n",stSol[1],stSol[4],stSol[7]);
		printf("%18.10f %18.10f %18.10f\n",stSol[2],stSol[5],stSol[8]);
	}
	else printf("Test %d frameSidereal OK!\n",testCount);

	double pm[9];
	framePolarm(jdSol, pm); // TEST
	testCount = testCount + 1;
	double pmSol[9] = {1.00000000000, -0.00000000000, -0.00000068205, 0.00000000000, 1.00000000000, -0.00000161593, 0.00000068205, 0.00000161593, 1.00000000000};
	same = 1;
	for (i=0; i<=8; i++){
		absError[i] = pm[i]-pmSol[i];
		relError[i] = absError[i] / pm[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("framePolarm error! pm not pmSol\n");
		printf("pm:\n%18.10f %18.10f %18.10f\n",pm[0],pm[3],pm[6]);
		printf("%18.10f %18.10f %18.10f\n",pm[1],pm[4],pm[7]);
		printf("%18.10f %18.10f %18.10f\n",pm[2],pm[5],pm[8]);
		printf("pmSol:\n%18.10f %18.10f %18.10f\n",pmSol[0],pmSol[3],pmSol[6]);
		printf("%18.10f %18.10f %18.10f\n",pmSol[1],pmSol[4],pmSol[7]);
		printf("%18.10f %18.10f %18.10f\n",pmSol[2],pmSol[5],pmSol[8]);
	}
	else printf("Test %d framePolarm OK!\n",testCount);

	double nutteme[9];
	frameTruemean(tttSol, nutteme); // TEST
	testCount = testCount + 1;
	double nuttemeSol[9] = {1.00000000000, 0.00000000897, 0.00000041328, -0.00000000897, 1.00000000000, -0.00000061887, -0.00000041328, 0.00000061887, 1.00000000000};
	same = 1;
	for (i=0; i<=8; i++){
		absError[i] = nutteme[i]-nuttemeSol[i];
		relError[i] = absError[i] / nutteme[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameTruemean error! nutteme not nuttemeSol\n");
		printf("nutteme:\n%18.10f %18.10f %18.10f\n",nutteme[0],nutteme[3],nutteme[6]);
		printf("%18.10f %18.10f %18.10f\n",nutteme[1],nutteme[4],nutteme[7]);
		printf("%18.10f %18.10f %18.10f\n",nutteme[2],nutteme[5],nutteme[8]);
		printf("nuttemeSol:\n%18.10f %18.10f %18.10f\n",nuttemeSol[0],nuttemeSol[3],nuttemeSol[6]);
		printf("%18.10f %18.10f %18.10f\n",nuttemeSol[1],nuttemeSol[4],nuttemeSol[7]);
		printf("%18.10f %18.10f %18.10f\n",nuttemeSol[2],nuttemeSol[5],nuttemeSol[8]);
	}
	else printf("Test %d frameTruemean OK!\n",testCount);

	double recefSol[3] = {-1033.4793830, 7901.2952754, 6380.3565958};
	double reci[3];
	frameEcef2eci(recefSol, timeSol, reci); // TEST
	testCount = testCount + 1;
	double reciSol[3] = {5102.5089579, 6123.0114007, 6378.1369282};
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = reci[i]-reciSol[i];
		relError[i] = absError[i] / reci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameEcef2eci error! reci not reciSol\n");
		printf("reci:   %18.10f %18.10f %18.10f\n",reci[0],reci[1],reci[2]);
		printf("reciSol:%18.10f %18.10f %18.10f\n",reciSol[0],reciSol[1],reciSol[2]);
	}
	else printf("Test %d frameEcef2eci OK!\n",testCount);

	double recef[3];
	frameEci2ecef(reciSol, timeSol, recef); // TEST
	testCount = testCount + 1;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = recef[i]-recefSol[i];
		relError[i] = absError[i] / recef[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameEci2ecef error! recef not recefSol\n");
		printf("recef:   %18.10f %18.10f %18.10f\n",recef[0],recef[1],recef[2]);
		printf("recefSol:%18.10f %18.10f %18.10f\n",recefSol[0],recefSol[1],recefSol[2]);
	}
	else printf("Test %d frameEci2ecef OK!\n",testCount);

	double rteme[3];
	frameEci2teme(reciSol, timeSol, rteme); // TEST
	testCount = testCount + 1;
	double rtemeSol[3] = {5094.1801621, 6127.6446595, 6380.3445327};
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = rteme[i]-rtemeSol[i];
		relError[i] = absError[i] / rteme[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameEci2teme error! rteme not rtemeSol\n");
		printf("rteme:   %18.10f %18.10f %18.10f\n",rteme[0],rteme[1],rteme[2]);
		printf("rtemeSol:%18.10f %18.10f %18.10f\n",rtemeSol[0],rtemeSol[1],rtemeSol[2]);
	}
	else printf("Test %d frameEci2teme OK!\n",testCount);

	frameTeme2eci(rtemeSol, timeSol, reci); // TEST
	testCount = testCount + 1;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = reci[i]-reciSol[i];
		relError[i] = absError[i] / reci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameTeme2eci error! reci not reciSol\n");
		printf("reci:   %18.10f %18.10f %18.10f\n",reci[0],reci[1],reci[2]);
		printf("reciSol:%18.10f %18.10f %18.10f\n",reciSol[0],reciSol[1],reciSol[2]);
	}
	else printf("Test %d frameTeme2eci OK!\n",testCount);

	// Excerpt of ex3_3.m from Vallado (2013) Fundamentals of Astrodynamics and Applications
	recefSol[0] = 6524.834;
	recefSol[1] = 6862.875;
	recefSol[2] = 6448.296;
	double rgeod[3];
	frameEcef2geod(recefSol, rgeod); // TEST
	testCount = testCount + 1;
	double rgeodSol[3] = {34.3524952, 46.4464169, 5085.2187362};
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = rgeod[i]-rgeodSol[i];
		relError[i] = absError[i] / rgeod[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameEcef2geod error! rgeod not rgeodSol\n");
		printf("rgeod:   %18.10f %18.10f %18.10f\n",rgeod[0],rgeod[1],rgeod[2]);
		printf("rgeodSol:%18.10f %18.10f %18.10f\n",rgeodSol[0],rgeodSol[1],rgeodSol[2]);
	}
	else printf("Test %d frameEcef2geod OK!\n",testCount);

	frameGeod2ecef(rgeodSol, recef); // TEST
	testCount = testCount + 1;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = recef[i]-recefSol[i];
		relError[i] = absError[i] / recef[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameGeod2ecef error! recef not recefSol\n");
		printf("recef:   %18.10f %18.10f %18.10f\n",recef[0],recef[1],recef[2]);
		printf("recefSol:%18.10f %18.10f %18.10f\n",recefSol[0],recefSol[1],recefSol[2]);
	}
	else printf("Test %d frameGeod2ecef OK!\n",testCount);

	// Verify frameRotationNed2ecef with the Marine Systems Simulator (MSS) toolbox
	double sinlat = sin(rgeodSol[0]);
	double coslat = cos(rgeodSol[0]);
	double sinlon = sin(rgeodSol[1]);
	double coslon = cos(rgeodSol[1]);
	double Ned2ecef[9];
	frameRotationNed2ecef(sinlat, coslat, sinlon, coslon, Ned2ecef); // TEST
	testCount = testCount + 1;
	double Ned2ecefSol[9] = {0.1586295368, -0.1276159144, -0.9790561008, -0.6268260173, -0.7791592546, 0.0000000000, -0.7628406217, 0.6136978363, -0.2035906470};
	same = 1;
	for (i=0; i<=8; i++){
		absError[i] = Ned2ecef[i]-Ned2ecefSol[i];
		relError[i] = absError[i] / Ned2ecef[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("frameRotationNed2ecef error! Ned2ecef not Ned2ecefSol\n");
		printf("Ned2ecef:\n%18.10f %18.10f %18.10f\n",Ned2ecef[0],Ned2ecef[3],Ned2ecef[6]);
		printf("%18.10f %18.10f %18.10f\n",Ned2ecef[1],Ned2ecef[4],Ned2ecef[7]);
		printf("%18.10f %18.10f %18.10f\n",Ned2ecef[2],Ned2ecef[5],Ned2ecef[8]);
		printf("Ned2ecefSol:\n%18.10f %18.10f %18.10f\n",Ned2ecefSol[0],Ned2ecefSol[3],Ned2ecefSol[6]);
		printf("%18.10f %18.10f %18.10f\n",Ned2ecefSol[1],Ned2ecefSol[4],Ned2ecefSol[7]);
		printf("%18.10f %18.10f %18.10f\n\n",Ned2ecefSol[2],Ned2ecefSol[5],Ned2ecefSol[8]);
	}
	else printf("Test %d frameRotationNed2ecef OK!\n\n",testCount);

/* Test the Sun Vector Prediction */
	// Excerpt of ex5_1.m from Vallado (2013) Fundamentals of Astrodynamics and Applications
	timeSol.year=2006;
	timeSol.mon = 4;
	timeSol.day = 2;
	timeSol.hr =  0;
	timeSol.min= 0;
	timeSol.sec= 0;
	double rsun[3];
	sun(timeSol, rsun); // TEST
	testCount = testCount + 1;
	double rsunSol[3] = {0.9780491365, 0.1911814374, 0.0828827162};
	double normRsunSol = hypot(hypot(rsunSol[0],rsunSol[1]), rsunSol[2]);
	rsunSol[0] = rsunSol[0]/normRsunSol;
	rsunSol[1] = rsunSol[1]/normRsunSol;
	rsunSol[2] = rsunSol[2]/normRsunSol;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = rsun[i]-rsunSol[i];
		relError[i] = absError[i] / rsun[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("sun error! rsun not rsunSol\n");
		printf("rsun:   %18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
		printf("rsunSol:%18.10f %18.10f %18.10f\n\n",rsunSol[0],rsunSol[1],rsunSol[2]);
	}
	else printf("Test %d sun OK!\n\n",testCount);

/* Test the Orbit Position and Velocity Prediction */
	// Original STR#3 SGP4 test from ValladoEtAl (2006) Revisiting Spacetrack Report no3
	char tleSol[139] = "1 88888U          80275.98708465  .00073094  13844-3  66816-4 0    872 88888  72.8435 115.9689 0086731  52.6988 110.5714 16.05824518  1058";
	Satrec satrec = orbitTwoline2rv(tleSol);
	printf("satrec.error after initialization: '%s'\n",satrec.error);
	timeSol.year=1980;
	timeSol.mon = 10;
	timeSol.day = 2;
	timeSol.hr =  1;
	timeSol.min= 41;
	timeSol.sec= 24.113771;
	double veci[3];
	orbitSgp4(&satrec, timeSol, reci, veci); // TEST // TEST
	printf("satrec.error after SGP4: '%s'\n",satrec.error);
	testCount = testCount + 1;
	reciSol[0] = 1022.4233730086;
	reciSol[1] = 2290.9626955932;
	reciSol[2] = -6189.6432266006;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = reci[i]-reciSol[i];
		relError[i] = absError[i] / reci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("orbitSgp4 position error! reci not reciSol\n");
		printf("reci:   %18.10f %18.10f %18.10f\n",reci[0],reci[1],reci[2]);
		printf("reciSol:%18.10f %18.10f %18.10f\n",reciSol[0],reciSol[1],reciSol[2]);
	}
	else printf("Test %d orbitSgp4 position OK!\n",testCount);
	double r = sqrt(absError[0]*absError[0] + absError[1]*absError[1] + absError[2]*absError[2]);
	double theta = acos(absError[2] / r);
	double phi = atan2(absError[1], absError[0]);
	printf("abs.sph.err.:%18.10f %18.10f %18.10f\n",r,theta,phi);
	testCount = testCount +1;
	double veciSol[3] = {-3.7777586815, 6.4513394443, 1.8209505936};
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = veci[i]-veciSol[i];
		relError[i] = absError[i] / veci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("orbitSgp4 velocity error! veci not veciSol\n");
		printf("veci:   %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
		printf("veciSol:%18.10f %18.10f %18.10f\n",veciSol[0],veciSol[1],veciSol[2]);
	}
	else printf("Test %d orbitSgp4 velocity OK!\n",testCount);

	// TEME example from Vallado (2013) Fundamentals of Astrodynamics and Applications p. 234
	char tleSol2[139] = "1 00005U 58002B   00179.78495062  .00000023  00000-0  28098-4 0  47532 00005  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413667";
	satrec = orbitTwoline2rv(tleSol2);
	printf("satrec.error after initialization: '%s'\n",satrec.error);
	timeSol.year=2000;
	timeSol.mon = 6;
	timeSol.day = 30;
	timeSol.hr =  18;
	timeSol.min= 50;
	timeSol.sec= 19.733571;
	orbitSgp4(&satrec, timeSol, reci, veci); // TEST // TEST
	printf("satrec.error after SGP4: '%s'\n",satrec.error);
	testCount = testCount + 1;
	reciSol[0] = -9059.9413786;
	reciSol[1] = 4659.6972000;
	reciSol[2] = 813.9588875;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = reci[i]-reciSol[i];
		relError[i] = absError[i] / reci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("orbitSgp4 position error! reci not reciSol\n");
		printf("reci:   %18.10f %18.10f %18.10f\n",reci[0],reci[1],reci[2]);
		printf("reciSol:%18.10f %18.10f %18.10f\n",reciSol[0],reciSol[1],reciSol[2]);
	}
	else printf("Test %d orbitSgp4 position OK!\n",testCount);
	r = sqrt(absError[0]*absError[0] + absError[1]*absError[1] + absError[2]*absError[2]);
	theta = acos(absError[2] / r);
	phi = atan2(absError[1], absError[0]);
	printf("abs.sph.err.:%18.10f %18.10f %18.10f\n",r,theta,phi);
	testCount = testCount +1;
	veciSol[0] = -2.233348094;
	veciSol[1] = -4.110136162;
	veciSol[2] = -3.157394074;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = veci[i]-veciSol[i];
		relError[i] = absError[i] / veci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("orbitSgp4 velocity error! veci not veciSol\n");
		printf("veci:   %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
		printf("veciSol:%18.10f %18.10f %18.10f\n\n",veciSol[0],veciSol[1],veciSol[2]);
	}
	else printf("Test %d orbitSgp4 velocity OK!\n\n",testCount);

/* Test the Geomagnetic Vector Prediction */
	// Official WMM 2015 test values
	timeSol.year=2015;
	timeSol.mon = 1;
	timeSol.day = 1;
	timeSol.hr =  0;
	timeSol.min= 0;
	timeSol.sec= 0.0;
	reciSol[0] = -186.3979236310;
	reciSol[1] = 1093.5121033253;
	reciSol[2] = 6259.8764001403;
	double Beci[3];
	start = clock(); // measure computation time
	magWmm(reciSol,timeSol,Beci); // TEST
	end = clock(); // measure computation time
	cpu_time_used = (end - start);
	testCount = testCount + 1;
	double BeciSol[3] = {3174.4799983588, -15647.9734763282, -52460.0435315694};
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magWmm error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magWmm OK!\n",testCount);
	printf("magWmm cpu_time_used first test:%d\n",cpu_time_used);

	// Magnetic Field Calculator http://www.ngdc.noaa.gov/geomag-web/?model=igrf#igrfwmm
	start = clock(); // measure computation time
	magIgrf(reciSol,timeSol,Beci); // TEST
	end = clock(); // measure computation time
	cpu_time_used = (end - start);
	testCount = testCount + 1;
	BeciSol[0] = 3176.4834170946;
	BeciSol[1] = -15651.8044943423;
	BeciSol[2] = -52461.5533707477;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magIgrf error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magIgrf OK!\n",testCount);
	printf("magIgrf cpu_time_used first test:%d\n",cpu_time_used);

	// Official WMM 2015 test values
	reciSol[0] = -4952.5257175308;
	reciSol[1] = -4175.9667490388;
	reciSol[2] = 7.0511621387;
	magWmm(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = -7946.3504665208;
	BeciSol[1] = -7225.2267426330;
	BeciSol[2] = 37546.9281707674;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magWmm error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magWmm OK!\n",testCount);

	// Magnetic Field Calculator http://www.ngdc.noaa.gov/geomag-web/?model=igrf#igrfwmm
	magIgrf(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = -7948.1795179286;
	BeciSol[1] = -7222.5871051027;
	BeciSol[2] = 37549.9309835582;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magIgrf error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magIgrf OK!\n",testCount);

	// Official WMM 2015 test values
	timeSol.year=2017;
	timeSol.mon = 7;
	timeSol.day = 3;
	timeSol.hr =  0;
	timeSol.min= 0;
	timeSol.sec= 0.0;
	reciSol[0] = 222.3186744526;
	reciSol[1] = -1091.0602700267;
	reciSol[2] = 6259.1315168516;
	magWmm(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = -3440.7027887425;
	BeciSol[1] = 15605.1788907721;
	BeciSol[2] = -52479.4917421705;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magWmm error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magWmm OK!\n",testCount);

	// Magnetic Field Calculator http://www.ngdc.noaa.gov/geomag-web/?model=igrf#igrfwmm
	magIgrf(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = -3440.1868719256;
	BeciSol[1] = 15606.2282269078;
	BeciSol[2] = -52475.5709356830;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magIgrf error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magIgrf OK!\n",testCount);

	// Official WMM 2015 test values
	reciSol[0] = 4890.0569722041;
	reciSol[1] = 4248.9453834772;
	reciSol[2] = -8.0577661365;
	magWmm(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = 7900.3409632929;
	BeciSol[1] = 7085.5154738892;
	BeciSol[2] = 37572.5445674497;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magWmm error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magWmm OK!\n",testCount);

	// Magnetic Field Calculator http://www.ngdc.noaa.gov/geomag-web/?model=igrf#igrfwmm
	magIgrf(reciSol,timeSol,Beci); // TEST
	testCount = testCount + 1;
	BeciSol[0] = 7906.7937764379;
	BeciSol[1] = 7094.7028926387;
	BeciSol[2] = 37570.0340815931;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magIgrf error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magIgrf OK!\n",testCount);

	// My birthday in three years 5km above Trondheim (lat=63, lon=10, alt=5)
	timeSol.year=2018;
	timeSol.mon = 10;
	timeSol.day = 15;
	timeSol.hr =  12;
	timeSol.min= 34;
	timeSol.sec= 56.789;
	reciSol[0] = -2132.3132036918;
	reciSol[1] = -1962.8765465817;
	reciSol[2] = 5668.2203696986;
	start = clock(); // measure computation time
	magWmm(reciSol,timeSol,Beci); // TEST
	end = clock(); // measure computation time
	cpu_time_used = (end - start);
	testCount = testCount + 1;
	BeciSol[0] = 26150.4541807207;
	BeciSol[1] = 22986.7162207562;
	BeciSol[2] = -38185.5741861825;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magWmm error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magWmm OK!\n",testCount);
	printf("magWmm cpu_time_used last test:%d\n",cpu_time_used);

	start = clock(); // measure computation time
	magIgrf(reciSol,timeSol,Beci); // TEST
	end = clock(); // measure computation time
	cpu_time_used = (end - start);
	testCount = testCount + 1;
	BeciSol[0] = 26166.5929731794;
	BeciSol[1] = 22972.4009119709;
	BeciSol[2] = -38220.2463841567;
	same = 1;
	for (i=0; i<=2; i++){
		absError[i] = Beci[i]-BeciSol[i];
		relError[i] = absError[i] / Beci[i];
		if (isinf(relError[i]) || isnan(relError[i])) relError[i] = 0.0;
		if ((fabs(absError[i]) > absTol) || (fabs(relError[i]) > relTol)) same = 0;
	}
	if (same == 0){
		errorCount = errorCount +1;
		printf("magIgrf error! Beci not BeciSol\n");
		printf("Beci:   %18.10f %18.10f %18.10f\n",Beci[0],Beci[1],Beci[2]);
		printf("BeciSol:%18.10f %18.10f %18.10f\n",BeciSol[0],BeciSol[1],BeciSol[2]);
	}
	else printf("Test %d magIgrf OK!\n",testCount);
	printf("magIgrf cpu_time_used last test:%d\n\n",cpu_time_used);

/* Test summary */
	printf("There is(are) %d error(s) in the %d verification tests.\n",errorCount,testCount);
}

void inexactClock(){
/* Define some Time variables */
	// The exact time that the clock should have (TLE epoch)
	Time exact;
	exact.year = 2015;
	exact.mon = 4;
	exact.day = 1;
	exact.hr = 4;
	exact.min = 2;
	exact.sec = 7.716768;
	// Inexact time, 10 seconds ahead
	Time inexact10sec;
	inexact10sec.year = 2015;
	inexact10sec.mon = 4;
	inexact10sec.day = 1;
	inexact10sec.hr = 4;
	inexact10sec.min = 2;
	inexact10sec.sec = 17.716768;
	// Inexact time, 30 seconds ahead
	Time inexact30sec;
	inexact30sec.year = 2015;
	inexact30sec.mon = 4;
	inexact30sec.day = 1;
	inexact30sec.hr = 4;
	inexact30sec.min = 2;
	inexact30sec.sec = 37.716768;
	// Inexact time, 1 minute ahead
	Time inexact1min;
	inexact1min.year = 2015;
	inexact1min.mon = 4;
	inexact1min.day = 1;
	inexact1min.hr = 4;
	inexact1min.min = 3;
	inexact1min.sec = 7.716768;
	// Inexact time, 2 minute ahead
	Time inexact2min;
	inexact2min.year = 2015;
	inexact2min.mon = 4;
	inexact2min.day = 1;
	inexact2min.hr = 4;
	inexact2min.min = 4;
	inexact2min.sec = 7.716768;
	// Inexact time, 3 minutes ahead
	Time inexact3min;
	inexact3min.year = 2015;
	inexact3min.mon = 4;
	inexact3min.day = 1;
	inexact3min.hr = 4;
	inexact3min.min = 5;
	inexact3min.sec = 7.716768;

/* Test the Sun Vector Prediction */
	double rsunSol[3] = {0.9780491365, 0.1911814374, 0.0828827162}; // Computed with code source
	printf("rsunSol:  %18.10f %18.10f %18.10f\n",rsunSol[0],rsunSol[1],rsunSol[2]);
	double rsun[3];
	sun(exact, rsun);
	printf("rsunExact:%18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
	sun(inexact10sec, rsun);
	printf("rsun10sec:%18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
	sun(inexact30sec, rsun);
	printf("rsun30sec:%18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
	sun(inexact1min, rsun);
	printf("rsun1min: %18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
	sun(inexact2min, rsun);
	printf("rsun2min :%18.10f %18.10f %18.10f\n",rsun[0],rsun[1],rsun[2]);
	sun(inexact3min, rsun);
	printf("rsun3min: %18.10f %18.10f %18.10f\n\n",rsun[0],rsun[1],rsun[2]);

/* Test the Orbit Position and Velocity Prediction */
	char tle[139] = "1 39446U 13066AG  15091.16814487  .00002750  00000-0  38274-3 0  99982 39446  97.7351 154.4636 0072683  33.0976 327.4752 14.76760372 71880";
	double reciSol[3] = {1022.4233730086, 2290.9626955932, -6189.6432266006};
	double rnorm = sqrt(reciSol[0]*reciSol[0] + reciSol[1]*reciSol[1] + reciSol[2]*reciSol[2]);
	printf("reciSol:   %18.10f %18.10f %18.10f\n",reciSol[0],reciSol[1],reciSol[2]);
	printf("rnormSol:  %18.10f\n",rnorm);
	double veciSol[3] = {-3.7777586815, 6.4513394443, 1.8209505936};
	double vnorm = sqrt(veciSol[0]*veciSol[0] + veciSol[1]*veciSol[1] + veciSol[2]*veciSol[2]);
	printf("veciSol:   %18.10f %18.10f %18.10f\n",veciSol[0],veciSol[1],veciSol[2]);
	printf("vnormSol:  %18.10f\n",vnorm);
	Satrec satrec = orbitTwoline2rv(tle);
	if (satrec.error[0] != '\0') printf("satrec.error after initialization: '%s'\n",satrec.error);
	double reciExact[3],reci10sec[3],reci30sec[3],reci1min[3],reci2min[3],reci3min[3]; // needed for geomagnetic field prediction
	double veci[3];
	orbitSgp4(&satrec, exact, reciExact, veci);
	rnorm = sqrt(reciExact[0]*reciExact[0] + reciExact[1]*reciExact[1] + reciExact[2]*reciExact[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reciExact: %18.10f %18.10f %18.10f\n",reciExact[0],reciExact[1],reciExact[2]);
	printf("rnormExact:%18.10f\n",rnorm);
	printf("veciExact: %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnormExact:%18.10f\n",vnorm);
	orbitSgp4(&satrec, inexact10sec, reci10sec, veci);
	rnorm = sqrt(reci10sec[0]*reci10sec[0] + reci10sec[1]*reci10sec[1] + reci10sec[2]*reci10sec[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reci10sec: %18.10f %18.10f %18.10f\n",reci10sec[0],reci10sec[1],reci10sec[2]);
	printf("rnorm10sec:%18.10f\n",rnorm);
	printf("veci10sec: %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnorm10sec:%18.10f\n",vnorm);
	orbitSgp4(&satrec, inexact30sec, reci30sec, veci);
	rnorm = sqrt(reci30sec[0]*reci30sec[0] + reci30sec[1]*reci30sec[1] + reci30sec[2]*reci30sec[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reci30sec: %18.10f %18.10f %18.10f\n",reci30sec[0],reci30sec[1],reci30sec[2]);
	printf("rnorm30sec:%18.10f\n",rnorm);
	printf("veci30sec: %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnorm30sec:%18.10f\n",vnorm);
	orbitSgp4(&satrec, inexact1min, reci1min, veci);
	rnorm = sqrt(reci1min[0]*reci1min[0] + reci1min[1]*reci1min[1] + reci1min[2]*reci1min[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reci1min:  %18.10f %18.10f %18.10f\n",reci1min[0],reci1min[1],reci1min[2]);
	printf("rnorm1min: %18.10f\n",rnorm);
	printf("veci1min:  %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnorm1min: %18.10f\n",vnorm);
	orbitSgp4(&satrec, inexact2min, reci2min, veci);
	rnorm = sqrt(reci2min[0]*reci2min[0] + reci2min[1]*reci2min[1] + reci2min[2]*reci2min[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reci2min:  %18.10f %18.10f %18.10f\n",reci2min[0],reci2min[1],reci2min[2]);
	printf("rnorm2min: %18.10f\n",rnorm);
	printf("veci2min:  %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnorm2min: %18.10f\n",vnorm);
	orbitSgp4(&satrec, inexact3min, reci3min, veci);
	rnorm = sqrt(reci3min[0]*reci3min[0] + reci3min[1]*reci3min[1] + reci3min[2]*reci3min[2]);
	vnorm = sqrt(veci[0]*veci[0] + veci[1]*veci[1] + veci[2]*veci[2]);
	if (satrec.error[0] != '\0') printf("satrec.error after SGP4: '%s'\n",satrec.error);
	printf("reci3min:  %18.10f %18.10f %18.10f\n",reci3min[0],reci3min[1],reci3min[2]);
	printf("rnorm3min: %18.10f\n",rnorm);
	printf("veci3min:  %18.10f %18.10f %18.10f\n",veci[0],veci[1],veci[2]);
	printf("vnorm3min: %18.10f\n\n",vnorm);

/* Test the Geomagnetic Vector Prediction */
	double WmmSol[3] = {3174.4799983588, -15647.9734763282, -52460.0435315694}; // Computed with code source and official calculator
	printf("WmmSol:  %18.10f %18.10f %18.10f\n",WmmSol[0],WmmSol[1],WmmSol[2]);
	double Wmm[3];
	magWmm(reciExact,exact,Wmm);
	printf("WmmExact:%18.10f %18.10f %18.10f\n",Wmm[0],Wmm[1],Wmm[2]);
	magWmm(reci10sec,inexact10sec,Wmm);
	printf("Wmm10sec:%18.10f %18.10f %18.10f\n",Wmm[0],Wmm[1],Wmm[2]);
	magWmm(reci30sec,inexact30sec,Wmm);
	printf("Wmm30sec:%18.10f %18.10f %18.10f\n",Wmm[0],Wmm[1],Wmm[2]);
	magWmm(reci1min,inexact1min,Wmm);
	printf("Wmm1min: %18.10f %18.10f %18.10f\n",Wmm[0],Wmm[1],Wmm[2]);
	magWmm(reci2min,inexact2min,Wmm);
	printf("Wmm2min: %18.10f %18.10f %18.10f\n",Wmm[0],Wmm[1],Wmm[2]);
	magWmm(reci3min,inexact3min,Wmm);
	printf("Wmm3min: %18.10f %18.10f %18.10f\n\n",Wmm[0],Wmm[1],Wmm[2]);

	double IgrfSol[3] = {3174.4799983588, -15647.9734763282, -52460.0435315694}; // Computed with code source and official calculator
	printf("IgrfSol:  %18.10f %18.10f %18.10f\n",IgrfSol[0],IgrfSol[1],IgrfSol[2]);
	double Igrf[3];
	magIgrf(reciExact,exact,Igrf);
	printf("IgrfExact:%18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
	magIgrf(reci10sec,inexact10sec,Igrf);
	printf("Igrf10sec:%18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
	magIgrf(reci30sec,inexact30sec,Igrf);
	printf("Igrf30sec:%18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
	magIgrf(reci1min,inexact1min,Igrf);
	printf("Igrf1min: %18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
	magIgrf(reci2min,inexact2min,Igrf);
	printf("Igrf2min: %18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
	magIgrf(reci3min,inexact3min,Igrf);
	printf("Igrf3min: %18.10f %18.10f %18.10f\n",Igrf[0],Igrf[1],Igrf[2]);
}

int test(){
	// Comment out the following two lines to suppress the verification tests.
	printf("____________________\n\nPredictor Verification Tests:\n\n");
	verification();

	// Comment out the following two lines to suppress the tests on the impact of an inexact clock on the predictions.
	printf("____________________\n\nImpact of an inexact Clock on the Predictions:\n\n");
	inexactClock();

	printf("____________________\n");
	return 0;
}
