/*
 * conf_csp_testing.h
 *
 *  Author: andrasg
 */ 

#ifndef CONF_CSP_TESTING_H_
#define CONF_CSP_TESTING_H_

/* The testing is made for a 2 device setup where
 * one is slave and one is initiator. This variable 
 * defines if the compilation will be for a slave or 
 * a initiator, 1 is initator, 0 is slave.
 */
#define IS_MODULE_8	0

//! The I2C and CSP address for the slave module
#define MODULE_4_ADDR 0x04
//! The I2C and CSP address for the initiator module
#define MODULE_8_ADDR 0x02
/* Shuffles the addresses around so they will be correct
 * relative to what model we are compiling for. i.e. for 
 * the slave module, the module 8 address will be slave address
 * also defines up some limiters that are used in the testing
 */
#if (IS_MODULE_8 == 0)
	#define TEST_SLAVE
	#define I2C_MASTER_ADDRESS	MODULE_4_ADDR
	#define I2C_SLAVE_ADDRESS	MODULE_8_ADDR
#else
	#define TEST_INITIATOR
	#define I2C_MASTER_ADDRESS	MODULE_8_ADDR
	#define I2C_SLAVE_ADDRESS	MODULE_4_ADDR
#endif

//! Number of idle increments to achieve before deciding the bus is free
#define I2C_IDLE_LIMIT 100
//! Number of busy increments to achieve before deciding the bus is busy
#define I2C_BUSY_LIMIT 300
//! Active port on the slave
#define SLAVE_PORT			0x12
//! Active port on the initiator
#define MASTER_PORT			0x13

/*Sizes*/
//! The size given by the overhead of a csp_packet in buffer
#define CSP_OVERHEAD_SIZE CSP_BUFFER_PACKET_OVERHEAD
//! The decided value of the maximum data a packet can contain
#define CSP_MAX_DATA_SIZE 12
//! A calculation of the maximum size of a csp packet, used as input to init buffer.
#define CSP_MAX_PACKET_SIZE (CSP_OVERHEAD_SIZE + CSP_MAX_DATA_SIZE)
//! Used in testing to decide the number of max size packets in init buffer.
#define CSP_NR_OF_BUFFERED_PACKETS 0x10
//! The minimum timeout for a transfer, since 0 is too short. Not used atm since timeout is not active.
#define CSP_MINIMUM_TRANSFER_TIMEOUT 200
//! A config for the test send and receive simultaneously on how many packets to send and receive.
#define TEST_SEND_RECV_AMOUNT 300
//! Listening on csp sockets, i use this value for the backlog queue length
//Backlog queue defines the number of pending connections on a socket
#define BACKLOG_CON_QUEUE 10

/*Some speed setup*/
//! The PBA speed input to the I2C inital setup functions.
#define I2C_PBA_SPEED		APPLI_PBA_SPEED
//! The baud rate the I2C bus should communicate over.
#define I2C_BAUDRATE		100000

/*Pins and functions to be enabled on an UC3-A3-256 board*/
#define I2C_CLK_PIN			AVR32_TWIMS0_TWCK_0_0_PIN
#define I2C_CLK_FUNCTION	AVR32_TWIMS0_TWCK_0_0_FUNCTION
#define I2C_DATA_PIN		AVR32_TWIMS0_TWD_0_0_PIN
#define I2C_DATA_FUNCTION	AVR32_TWIMS0_TWD_0_0_FUNCTION

/* IRQ and Module for both Master and slave, and the clock that drives the master bus, not the speed of it */
#define I2C_PBA_CLK			AVR32_TWIM0_CLK_PBA
#define I2C_MASTER_IRQ		AVR32_TWIM0_IRQ
#define I2C_MASTER_MODULE	AVR32_TWIM0
#define I2C_SLAVE_IRQ		AVR32_TWIS0_IRQ
#define I2C_SLAVE_MODULE	AVR32_TWIS0

/*CSP defines*/
//! Define used in csp framework to symbolize the maximum number of bytes that are able to be transfered.
#define I2C_MTU CSP_MAX_DATA_SIZE 
//! A no error status CSP uses in I2C interface.
#define E_NO_ERR STATUS_OK 

#define configTSK_LISTENTASK_STACK_SIZE		1024

//! The name given to every listener task that is run
#define configTSK_DEBUGTASK_NAME			((const signed portCHAR *) "Debug-task")
//! Stacksize for every listener task that is run
#define configTSK_DEBUGTASK_STACK_SIZE		1024
//! Priority for every listener task that is run
#define configTSK_DEBUGTASK_PRIORITY		(tskIDLE_PRIORITY + 3)

//! The name given to every listener task that is run
#define configTSK_DEBUGTASK2_NAME			((const signed portCHAR *) "Debug-task2")
//! Stacksize for every listener task that is run
#define configTSK_DEBUGTASK2_STACK_SIZE		1024
//! Priority for every listener task that is run
#define configTSK_DEBUGTASK2_PRIORITY		(tskIDLE_PRIORITY + 2)

//! Stacksize of the csp routing task
#define configTSK_ROUTINGTASK_STACK_SIZE	1024
//! Priority of the csp routing task
#define configTSK_ROUTINGTASK_PRIORITY		(tskIDLE_PRIORITY)

//! Name given to every test that is run when running
#define configTSK_TEST_TASK_NAME			((const signed portCHAR *) "Test-task")
//! Name given to every task executed by the listener
#define configTSK_LISTENER_TASK_NAME			((const signed portCHAR *) "Listener-task")
//! Stacksize for every test that is run
#define configTSK_TEST_TASK_STACK_SIZE		1024
//! Priority of every test that is run
#define configTSK_TEST_TASK_PRIORITY		(tskIDLE_PRIORITY + 2)

#define configTSK_LISTEN_TASK_PRIORITY		(tskIDLE_PRIORITY + 3)

#endif /* CONF_CSP_TESTING_H_ */