/*
 * nrp_ack.c
 *
 * Created: 20-Mar-15 2:05:41 PM
 *  Author: Erlend
 */ 
#include "csplib/NRP/nrp_ack.h"
#include "csplib/NRP/nrp.h"
#include "csplib/NRP/nrp_header.h"
#include "csplib/NRP/nrp_conn.h"
#include "csplib/NRP/nrp_debug.h"
#include "csplib/include/csp/csp_buffer.h"
#include "csplib/include/csp/csp.h"
#include "csplib/NRP/nrp_error.h"
#include "nrp_tester.h"
#include "semphr.h"
#include <stdbool.h>
#include <stdlib.h>

int8_t nrp_acknowledge_packet(const csp_packet_t *csp_packet, uint32_t next_expected_seq_num, uint8_t flags, int8_t additional_info){
	//create temporary destination socket
	module_socket_t *dest;
	if ((dest = pvPortMalloc(sizeof(*dest))) == NULL){
		NRP_PRINT_ERROR("malloc failed");
		return MALLOC_FAILED;
	}
	dest->csp_address = csp_packet->id.src;
	dest->port = csp_packet->id.sport;
	
	//create temporary source socket
	module_socket_t *src;
	if ((src = pvPortMalloc(sizeof(*src))) == NULL){
		NRP_PRINT_ERROR("malloc failed");
		vPortFree(dest);
		return MALLOC_FAILED;
	}
	src->csp_address = nuts_modules[this_module].csp_address;
	src->port = csp_packet->id.dport;
	
	//csp "dummy" header to generate checksum
	uint8_t csp_dummy_header[CSP_HEADER_LENGTH];
	make_csp_dummy_header(csp_dummy_header, dest, src, csp_packet->id.pri);

	
	csp_packet_t *out_packet = csp_buffer_get_wrapper(sizeof(csp_packet_t) + NRP_HEADER_SIZE + (flags == NRP_SYN_NAK ? 1 : 0));
	if (out_packet == NULL){
		NRP_PRINT_ERROR("CSP buffer malloc failed");
		vPortFree(src);
		vPortFree(dest);
		return MALLOC_FAILED;
	}
	// add one payload byte if syn_nak
	out_packet->length = NRP_HEADER_SIZE + (flags == NRP_SYN_NAK ? 1 : 0);
	
	uint8_t ack_packet[NRP_HEADER_SIZE + (flags == NRP_SYN_NAK ? 1 : 0)];
	if (flags == NRP_SYN_NAK){
		nrp_make_SYN_NAK_packet(ack_packet, csp_dummy_header, next_expected_seq_num, additional_info);
	}
	else {
		nrp_make_ACK_packet(ack_packet, csp_dummy_header, next_expected_seq_num, flags);
	}
	
	#if NRP_TESTING == true
	uint16_t rando = rand() % 1000;
	if (rando < (uint16_t)(NRP_TEST_PACKET_ERROR_PROB * 10)) {
		flip_random_bits(ack_packet, (rand() % 10) + 1);
		NRP_PRINT_WARNING("Sending corrupted acknowledgement");
		//corrupted = true;
		//corrupted_packets++;
		//printf("Corrupted packets: %i\r\n", corrupted_packets);
	}
	#endif
	
	memcpy(out_packet->data, ack_packet, NRP_HEADER_SIZE + (flags == NRP_SYN_NAK ? 1 : 0));

	NRP_PRINT("Senfding acknowledgement");
	//All is good, proceed with sending acknowledgement!
	int8_t a_status = csp_sendto(csp_packet->id.pri, dest->csp_address, dest->port, src->port, (uint32_t)NULL, out_packet, (uint32_t)NULL);
	if (a_status == CSP_ERR_NOMEM){
		//free packet from csp_buffer if not sent
		NRP_PRINT_WARNING("csp_send() returned error code");
	}
	csp_buffer_free(out_packet);
	//vTaskDelay(10);
	
	//free memory
	vPortFree(src);
	vPortFree(dest);
	return SUCCESS;
}
