/*
 * nrp_ack.h
 *
 * Created: 20-Mar-15 2:05:54 PM
 *  Author: Erlend Riis Jahren
 */ 


#ifndef NRP_ACK_H_
#define NRP_ACK_H_

#include "csplib/NRP/nrp_header.h"


#define ACK_RETRANS_DELAY			( TickType_t ) 2000

typedef struct {
	uint8_t pri;
	uint8_t dest;
	uint8_t dport;
	uint8_t sport;
	uint8_t data[NRP_HEADER_SIZE]; //NRP packet, no payload in ack packets
	uint64_t last_trans_tick_time;
	int8_t retrans_left;
}nrp_wait_for_ack_t;

/**
 * The function sends an acknowledgement packet to the transmitter of a csp packet.
 * Any type of csp_packet (different flags) can be passed to the function, and an appropriate
 * acknowledgement will be sent
 *
 * @param *csp_packet:						a pointer to the packet that needs acknowledgement
 * @param next_expected_seq_num				the next expected sequence number of the stream
 * @param flags								the flags in the packet
 * @param additional_info					used for the error_code in a SYN_NAK packet. could be used for more in later revisions
 *
 * @return:									SUCCESS if successful, negative error code if not (see nrp_error.h)
 */
int8_t nrp_acknowledge_packet(const csp_packet_t *csp_packet, uint32_t next_expected_seq_num, uint8_t flags, int8_t additional_info);

#endif /* NRP_ACK_H_ */