/*
 * nrp_csp_if.h
 *
 * Created: 20-Feb-15 2:11:05 PM
 *  Author: Erlend Riis Jahren
 *
 * Interface between NRP and CSP
 */ 


#ifndef NRP_CSP_IF_H_
#define NRP_CSP_IF_H_


/**
 * Creates a listener socket on a port by calling the csp_socket function with the
 * socket_opt parameter
 *
 * @param port:			The port to listen to (use CSP_ANY to listen to all ports)
 * @param socket_opt:	Specify the socket options of the listening port, see csp socket options
 *
 * @return:				A pointer to the socket if successful, NULL if not
 */
csp_socket_t * create_listen_socket(uint8_t port, uint32_t socket_opt);


/**
 * Creates a FreeRTOS task running the function given by the pointer as parameter
 * The task allocates memory of size configTSK_LISTENTASK_STACK_SIZE
 *
 * @param listen_port:		The port the task will be listening to. The port number is passed as
 *							as an argument to the function ran by the task
 * @param *listener_ptr:	A pointer to the function the task will run
 *
 * @return:					SUCCESS if the task is successfully started, NRP error code if not
 */
int8_t csp_create_listener(uint8_t listen_port, void (*listener_ptr)(void *arg_ptr));




#endif /* NRP_CSP_IF_H_ */