/*
 * nrp_debug.c
 *
 * Created: 20-Mar-15 2:10:19 PM
 *  Author: Erlend
 */ 

#include "csplib/NRP/nrp_debug.h"
#include "csplib/NRP/nrp_header.h"
#include "csplib/NRP/nrp_error.h"
#include <ctype.h>

/************************************************************************/
/* Test Function - only ran when debugging                              */
/************************************************************************/

void string_toupper(char *str){
	int i = 0;
	while (str[i] != '\0'){
		str[i] = toupper(str[i++]);
	}
}

void print_packet(uint8_t packet[]){
	#if NRP_DEBUG >= NRP_DEBUG_ALL
	//Extract payload length from packet
	uint8_t payload_length = packet[NRP_PAYLOAD_LENGTH_BYTE];
	
	NRP_PRINT_SUCCESS("---------------------   packet   ---------------------");
	print_packet_info(packet);
	printf("Header: ");
	for (int i = 0; i < NRP_HEADER_SIZE_WITHOUT_CRC; i++){ //Checksum extracted from header
		printbincharpad(packet[i]);
		printf(" ");
	}
	printf("\r\nSeq/ack: %i\r\n", (packet[NRP_SEQ_BYTE_FIRST] << NRP_SEQ_BYTE_FIRST_SHIFT) | (packet[NRP_SEQ_BYTE_SECOND] << NRP_SEQ_BYTE_SECOND_SHIFT) | (packet[NRP_SEQ_BYTE_THIRD] << NRP_SEQ_BYTE_THIRD_SHIFT));
	fflush(stdout);
	
	printf("\r\n\nPayload:\r\n");
	for (int i = NRP_HEADER_SIZE_WITHOUT_CRC; i < NRP_HEADER_SIZE_WITHOUT_CRC + payload_length; i++){ //Checksum extracted from header
		printf("Byte %i: 	", i - (NRP_HEADER_SIZE_WITHOUT_CRC) );
		printbincharpad(packet[i]);
		printf("  -  '%c'\n\r", (char)packet[i]);
	}
	
	printf("\r\nChecksum: ");
	printbincharpad(packet[NRP_HEADER_SIZE_WITHOUT_CRC + payload_length]);
	printf(" ");
	printbincharpad(packet[NRP_HEADER_SIZE_WITHOUT_CRC + 1 + payload_length]);
	printf("\r\n");
	
	
	NRP_PRINT_SUCCESS("------------------------------------------------------");
	vTaskDelay(50);
	fflush(stdout);
	
	#endif
}
void printbincharpad(char c)
{
	#if NRP_DEBUG >= NRP_DEBUG_ALL
	for (int i = 7; i >= 0; --i)
	{
		putchar( (c & (1 << i)) ? '1' : '0' );
	}
	vTaskDelay(10);
	#endif
}

void nrp_print_error(char *out, int line_number, const char *function_name){
	#if NRP_DEBUG >= NRP_DEBUG_E
	printf("\r%s%s:line %i>%s %s%s%s\r\n",BOLD_WHITE,function_name, line_number, WHITE, RED, out, WHITE);
	vTaskDelay(10);
	#endif
}

int8_t nrp_print_error_code(int8_t err_code, int line_number, const char *function_name){
	#if NRP_DEBUG >= NRP_DEBUG_E
	char buf[50];
	nrp_get_error_message(buf, err_code);
	printf("\r%s%s:line %i>%s %s%s%s\r\n",BOLD_WHITE,function_name, line_number, WHITE, RED, buf, WHITE);
	vTaskDelay(10);
	#endif
	
	return err_code; 
}

void nrp_print_warning(char *out, int line_number, const char *function_name){
	#if NRP_DEBUG >= NRP_DEBUG_EW
	printf("\r%s%s:line %i>%s %s%s%s\r\n",BOLD_WHITE,function_name,line_number,WHITE, YELLOW, out, WHITE);
	vTaskDelay(10);
	fflush(stdout);
	#endif
}

void nrp_print_success(char *out){
	#if NRP_DEBUG >= NRP_DEBUG_EWS
	printf("\r%s%s%s\r\n", GREEN, out, WHITE);
	vTaskDelay(10);
	#endif
}

void nrp_print(char *out){
	#if NRP_DEBUG >= NRP_DEBUG_ALL
	printf("\r%s\r\n", out);
	vTaskDelay(10);
	#endif
}


void print_packet_info(uint8_t *packet){
	
	#if NRP_DEBUG >= NRP_DEBUG_ALL
	
	uint8_t flags = packet[NRP_FLAG_BYTE];
	/*Check for flags*/
	switch (flags){
		case NRP_SYN:
		NRP_PRINT("flags: SYN");
		break;
		
		case NRP_SYN_ACK:
		NRP_PRINT("flags: SYN_ACK");
		break;
		
		case NRP_SYN_NAK:
		NRP_PRINT_WARNING("flags: SYN_NAK");
		break;
		
		case NRP_ARQ:
		NRP_PRINT("flags: ARQ (content packet)");
		break;
		
		case NRP_ACK:
		NRP_PRINT("flags: ACK");
		break;
		
		case NRP_FIN:
		NRP_PRINT("flags: FIN");
		break;
		
		case NRP_FIN_ACK:
		NRP_PRINT("flags: FIN_ACK");
		break;
		
		case NRP_NAK:
		NRP_PRINT_WARNING("flags: NAK");
		break;
		
		default:
		NRP_PRINT_WARNING("Flags not recognized");
	}

	#endif
}