/*
 * nrp_debug.h
 *
 * Created: 20-Mar-15 2:10:29 PM
 *  Author: Erlend
 */ 

#include "csplib/NRP/nrp.h"

#ifndef NRP_DEBUG_H_
#define NRP_DEBUG_H_

/* Debug output levels */
#define NRP_DEBUG_OFF			0
#define NRP_DEBUG_E				1
#define NRP_DEBUG_EW			2
#define NRP_DEBUG_EWS			3
#define NRP_DEBUG_ALL			4



/* Colors for printing. Turn off by setting NRP_DEBUG_USE_ANSI to false in nrp.h */
#if NRP_DEBUG_USE_ANSI == true
	static const char WHITE[]		= "\033[0m";
	static const char BOLD_WHITE[]	= "\033[1m";
	static const char RED[]			= "\033[91m";
	static const char GREEN[]		= "\033[92m";
	static const char YELLOW[]		= "\033[93m";
	static const char BLUE[]		= "\033[94m";
#else
	static const char WHITE[]		= "";
	static const char BOLD_WHITE[]	= "";
	static const char RED[]			= "";
	static const char GREEN[]		= "";
	static const char YELLOW[]		= "";
	static const char BLUE[]		= "";
#endif



/************************************************************************/
/* Macros for debugging with function names and line number				*/
/************************************************************************/

//Prints the "out" string parameter with function name and line number (red colored)
#define NRP_PRINT_ERROR(out...) nrp_print_error(out, __LINE__, __FUNCTION__)

//Prints an error msg depening on the NRP error code set by the "out" parameter (red colored)
#define NRP_PRINT_ERROR_CODE(out...) nrp_print_error_code(out, __LINE__, __FUNCTION__)

//Prints the "out" string parameter with function name and line number (yellow colored)
#define NRP_PRINT_WARNING(out...) nrp_print_warning(out, __LINE__, __FUNCTION__)

//Prints the "out" string parameter (green colored)
#define NRP_PRINT_SUCCESS(out...) nrp_print_success(out)

//Prints the "out" string parameter (white colored)
#define NRP_PRINT(out...) nrp_print(out)



/************************************************************************/
/* Functions			                                                */
/************************************************************************/
/*	These functions are only useful when debugging						*/
/*	Use the macros given above instead of direct function calls			*/
/************************************************************************/

void string_toupper(char *str);

void print_packet(uint8_t packet[]);

void printbincharpad(char c);

void nrp_print_error(char *out, int line_number, const char *function_name);

int8_t nrp_print_error_code(int8_t out, int line_number, const char *function_name);

void nrp_print_warning(char *out, int line_number, const char *function_name);

void nrp_print_success(char *out);

void nrp_print(char *out);

void print_packet_info(uint8_t *packet);


#endif /* NRP_DEBUG_H_ */