/*
 * nrp_header.c
 *
 * Created: 16-Feb-15 2:55:34 PM
 *  Author: Erlend Riis Jahren
 */ 

#include "nrp.h"
#include "nrp_header.h"
#include "crc.h"

#define BITS_IN_BYTE	8


void nrp_make_FIN_packet(uint8_t rel_packet[NRP_HEADER_SIZE], uint8_t csp_header[CSP_HEADER_LENGTH]){
	rel_packet[NRP_FLAG_BYTE] =  NRP_FIN;
	for (int i = NRP_SEQ_BYTE_FIRST; i < NRP_HEADER_SIZE_WITHOUT_CRC; i++) rel_packet[i] = 0;
	
	//Calculate checksum and add it to the packet (last two bytes)
	int full_packet_length = CSP_HEADER_LENGTH + NRP_HEADER_SIZE_WITHOUT_CRC;
	uint8_t full_packet[full_packet_length];
	memcpy(&full_packet[0], csp_header, CSP_HEADER_LENGTH * sizeof(uint8_t));
	memcpy(&full_packet[CSP_HEADER_LENGTH], rel_packet, NRP_HEADER_SIZE_WITHOUT_CRC * sizeof(uint8_t));
	
	unsigned short crc_bytes = crcFast(full_packet, full_packet_length);
	rel_packet[NRP_PAYLOAD_START_BYTE] = (crc_bytes >> BITS_IN_BYTE) & 0xff;
	rel_packet[NRP_PAYLOAD_START_BYTE + 1] = crc_bytes & 0xff;
}

void nrp_make_ACK_packet(uint8_t rel_packet[NRP_HEADER_SIZE], uint8_t csp_header[CSP_HEADER_LENGTH], uint32_t ack_num, uint8_t flags){
	rel_packet[NRP_FLAG_BYTE] =  flags;
	
	//set ack number
	rel_packet[NRP_SEQ_BYTE_FIRST] = ((ack_num & 0x00ff0000) >> NRP_SEQ_BYTE_FIRST_SHIFT);
	rel_packet[NRP_SEQ_BYTE_SECOND] = ((ack_num & 0x0000ff00) >> NRP_SEQ_BYTE_SECOND_SHIFT);
	rel_packet[NRP_SEQ_BYTE_THIRD] = ((ack_num & 0x000000ff) >> NRP_SEQ_BYTE_THIRD_SHIFT);
	
	rel_packet[NRP_PAYLOAD_LENGTH_BYTE] = 0;
	
	//Calculate checksum and add it to the packet (last two bytes)
	int full_packet_length = CSP_HEADER_LENGTH + NRP_HEADER_SIZE_WITHOUT_CRC;
	uint8_t full_packet[full_packet_length];
	memcpy(&full_packet[0], csp_header, CSP_HEADER_LENGTH * sizeof(uint8_t));
	memcpy(&full_packet[CSP_HEADER_LENGTH], rel_packet, NRP_HEADER_SIZE_WITHOUT_CRC * sizeof(uint8_t));
	
	unsigned short crc_bytes = crcFast(full_packet, full_packet_length);
	rel_packet[NRP_PAYLOAD_START_BYTE] = (crc_bytes >> BITS_IN_BYTE) & 0xff;
	rel_packet[NRP_PAYLOAD_START_BYTE + 1] = crc_bytes & 0xff;
}

void nrp_make_SYN_NAK_packet(uint8_t rel_packet[NRP_HEADER_SIZE], uint8_t csp_header[CSP_HEADER_LENGTH], uint32_t ack_num, int8_t error_byte){
	rel_packet[NRP_FLAG_BYTE] =  NRP_SYN_NAK;
	
	//set ack number
	rel_packet[NRP_SEQ_BYTE_FIRST] = ((ack_num & 0x00ff0000) >> NRP_SEQ_BYTE_FIRST_SHIFT);
	rel_packet[NRP_SEQ_BYTE_SECOND] = ((ack_num & 0x0000ff00) >> NRP_SEQ_BYTE_SECOND_SHIFT);
	rel_packet[NRP_SEQ_BYTE_THIRD] = ((ack_num & 0x000000ff) >> NRP_SEQ_BYTE_THIRD_SHIFT);
	
	rel_packet[NRP_PAYLOAD_LENGTH_BYTE] = 1; //the error byte
	
	rel_packet[NRP_PAYLOAD_START_BYTE] = error_byte;
	
	//Calculate checksum and add it to the packet (last two bytes)
	int full_packet_length = CSP_HEADER_LENGTH + NRP_HEADER_SIZE_WITHOUT_CRC + 1;
	uint8_t full_packet[full_packet_length];
	memcpy(&full_packet[0], csp_header, CSP_HEADER_LENGTH * sizeof(uint8_t));
	memcpy(&full_packet[CSP_HEADER_LENGTH], rel_packet, (NRP_HEADER_SIZE_WITHOUT_CRC + 1) * sizeof(uint8_t));
	
	unsigned short crc_bytes = crcFast(full_packet, full_packet_length);
	rel_packet[NRP_PAYLOAD_START_BYTE + 1] = (crc_bytes >> BITS_IN_BYTE) & 0xff;
	rel_packet[NRP_PAYLOAD_START_BYTE + 2] = crc_bytes & 0xff;
}

void nrp_make_reliable_packet(uint8_t rel_packet[], uint8_t flags, uint32_t seq_ack_num, 
								const uint8_t payload[], uint8_t payload_length, 
								uint8_t csp_header[4]){
	
	rel_packet[NRP_FLAG_BYTE] =  flags;
	
	//Use next three bytes as sequence number field
	rel_packet[NRP_SEQ_BYTE_FIRST] = ((seq_ack_num & 0x00ff0000) >> NRP_SEQ_BYTE_FIRST_SHIFT);
	rel_packet[NRP_SEQ_BYTE_SECOND] = ((seq_ack_num & 0x0000ff00) >> NRP_SEQ_BYTE_SECOND_SHIFT);
	rel_packet[NRP_SEQ_BYTE_THIRD] = ((seq_ack_num & 0x000000ff) >> NRP_SEQ_BYTE_THIRD_SHIFT);
	
	rel_packet[NRP_PAYLOAD_LENGTH_BYTE] = payload_length;
	
	//copy data into payload field of packet
	for (int i = 0; i < payload_length; i++)
		rel_packet[NRP_PAYLOAD_START_BYTE + i] = *(payload++); //Add payload to array byte for byte
	
	//Calculate checksum and add it to the packet (last two bytes)
	int full_packet_length = CSP_HEADER_LENGTH + NRP_HEADER_SIZE_WITHOUT_CRC + payload_length;
	uint8_t full_packet[full_packet_length];
	memcpy(&full_packet[0], csp_header, CSP_HEADER_LENGTH * sizeof(uint8_t));
	memcpy(&full_packet[CSP_HEADER_LENGTH], rel_packet, NRP_HEADER_SIZE_WITHOUT_CRC + payload_length * sizeof(uint8_t));
	
	unsigned short crc_bytes = crcFast(full_packet, full_packet_length);
	rel_packet[NRP_PAYLOAD_START_BYTE + payload_length] = (crc_bytes >> BITS_IN_BYTE) & 0xff;
	rel_packet[NRP_PAYLOAD_START_BYTE + payload_length + 1] = crc_bytes & 0xff;
	
}


void nrp_make_unreliable_packet(uint8_t unrel_packet[], const uint8_t payload[], uint8_t payload_length, uint8_t csp_header[4]){
	unrel_packet[NRP_FLAG_BYTE] = NRP_NO_FLAGS;
	
	for (uint8_t i = 1; i < NRP_PAYLOAD_START_BYTE; i++) unrel_packet[i] = '\0';
	
	//Calculate checksum and add it to the packet (last two bytes)
	int full_packet_length = CSP_HEADER_LENGTH + NRP_HEADER_SIZE_WITHOUT_CRC + payload_length;
	uint8_t full_packet[full_packet_length];
	memcpy(full_packet, csp_header, CSP_HEADER_LENGTH * sizeof(uint8_t));
	memcpy(&full_packet[CSP_HEADER_LENGTH], unrel_packet, (NRP_HEADER_SIZE_WITHOUT_CRC + payload_length) * sizeof(uint8_t));
	
	unsigned short crc_bytes = crcFast(full_packet, full_packet_length);
	unrel_packet[NRP_PAYLOAD_START_BYTE + payload_length] = (crc_bytes >> BITS_IN_BYTE) & 0xff;
	unrel_packet[NRP_PAYLOAD_START_BYTE + payload_length + 1] = crc_bytes & 0xff;
}


/*Hardcoded and ugly, but it follows the CSP protocol header v.1.0+ which can be found at:
 *http://en.wikipedia.org/wiki/Cubesat_Space_Protocol
 */
void make_csp_dummy_header(uint8_t csp_header[CSP_HEADER_LENGTH], const module_socket_t *dest, const module_socket_t *source, csp_prio_t priority){
	csp_header[0] = ((priority & 0x03) << 6) | ((source->csp_address & 0x1f) << 1) | ((dest->csp_address & 0x10) >> 4);
	csp_header[1] = ((dest->csp_address & 0x0f) << 4) | ((dest->port & 0x3c) >> 2);
	csp_header[2] = ((dest->port & 0x03) << 6) | (source->port & 0x3f);
	csp_header[3] = 0; //All csp flags are disabled
}