/*
 * nrp_tester.c
 *
 * Created: 14-Apr-15 6:12:50 PM
 *  Author: Erlend Riis Jahren
 */ 
#include "nrp_tester.h"
#include "nrp_header.h"
#include "nrp_debug.h"

/*
	Run tests for:
	
	 - long run test about 12/24 or hours or more without fails
	 - test with multiple connections simultanously
	 - Speed tests with different window sizes
	 - speed test with multiple connections
	 
	 - python code for testing how much a selective repeat upgrade can speed up the solution
*/


typedef struct {
	uint8_t byte;
	uint8_t bit;	
}test_bit_flip_t;


void flip_random_bits(uint8_t packet[], uint8_t number_of_bitflips){
	uint8_t length = NRP_HEADER_SIZE + packet[NRP_PAYLOAD_LENGTH_BYTE];
	test_bit_flip_t flipped[number_of_bitflips];
	for (int i = 0; i < number_of_bitflips; i++){
		uint8_t in_byte = rand() % length;
		uint8_t bit = rand() % 8;
		Bool already_flipped = false;
		//check that the bit is not already flipped
		for (int j = 0; j < i; j++){
			if (flipped[j].byte == in_byte && flipped[j].bit == bit){
				i--; 
				already_flipped = true;
			}
		}
		
		if (!already_flipped){
			packet[in_byte] ^= 1 << bit;  //flip bit with XOR
			flipped[i].byte = in_byte;
			flipped[i].bit = bit;
			//char buf[20];
			//sprintf(buf, "Flipped bit %i in byte %i", bit, in_byte);
			//NRP_PRINT(buf);
		}
	}
} 


void print_test_struct(nrp_test_t *test_struct, uint16_t tries){
	printf("Tries: %i", tries);
	printf("\rAvg time: %f\r\n", (float)((float)test_struct->tot_time / (float)tries));
	printf("total time: %i\r\n", test_struct->tot_time);
	printf("Avg packet loss per stream: %f\r\n", (float)((float)test_struct->tot_packet_loss / (float)(tries - test_struct->trans_fails)));
	printf("total packet loss (only succeeded trans): %i\r\n", test_struct->tot_packet_loss);	
	printf("packet loss percent: %f\r\n", (float)((float)test_struct->tot_packet_loss / (float)((tries - test_struct->trans_fails)*100)));
	printf("Trans fails: %i\r\n", test_struct->trans_fails);
	printf("Valid tries: %i\r\n", tries - test_struct->trans_fails);
	
	
}