/*
 * csp_example.c
 *
 * Created: 12-Nov-14 12:05:48 PM
 *  Author: Erlend
 */ 

#include "csp_example.h"
#include "csplib/NRP/nrp_debug.h"
#include "NRP/nrp.h"
#include "NRP/nrp_error.h"
#include <string.h>
#include "portmacro.h"
#include "FreeRTOS.h"
#include "task.h"
#include "serial/serial.h"
#include "cli/cli.h"

#include <stdlib.h>

#include <stdio.h>
#include <errno.h>


#include "csplib/NRP/nrp_tester.h"

#include <inttypes.h>

/*Packet examples - Allowed size range from 4 to 1200 characters */
static char *ping_packet = "PING";
static char *big_packet = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz01";

static const char *huge_packet = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz012345";

int packet_counter_success;



void example_listen_task(void *ptr);
void example_printout_listen_task(void *ptr);

void example_setup_of_listener(portCHAR * listen_port){
	uint8_t i = atoi(listen_port);
	int status = nrp_create_port_handler(i, &example_listen_task, 1024 + 256, tskIDLE_PRIORITY);
	if (status != pdPASS){
		printf("\rError creating listener: status %i\r\n", status);
	}
	printf("Heap left: %i\r\n", xPortGetFreeHeapSize());
}

void example_setup_of_printout_listener(portCHAR * listen_port){
	uint8_t i = atoi(listen_port);
	int status = nrp_create_port_handler(i, &example_printout_listen_task, 1024 + 256, tskIDLE_PRIORITY);
	if (status != pdPASS){
		printf("\rError creating listener: status %i\r\n", status);
		
	}
	printf("Heap left: %i\r\n", xPortGetFreeHeapSize());
	
}


void example_test(void *ptr){
	char *message = big_packet;
	nuts_module_t to_module = nuts_modules[0];
	
	uint8_t pri;
	if (nuts_modules[this_module].csp_address == 3){
		vTaskDelay(20000);
		pri = CSP_PRIO_CRITICAL;
	}
	else pri = CSP_PRIO_NORM;
	module_socket_t to_socket = {to_module.csp_address, 10};
	module_socket_t from_socket = {nuts_modules[this_module].csp_address, 10};
	nrp_test_t test_s;
	test_s.tot_packet_loss = 0;
	test_s.tot_time = 0;
	test_s.trans_fails = 0;
	uint32_t last_time;
	uint32_t avg_time = 0;
	uint16_t tries = 1000;
	uint32_t time;
	uint32_t fail = 0;
	uint64_t start = xTaskGetTickCount();
	printf("start=%" PRIu64 "\r\n", start);
	for (int i = 0; i < tries + fail; i++){
		last_time = xTaskGetTickCount();
		int8_t status = nrp_send(&to_socket, &from_socket, (uint8_t*) message, strlen(message), pri);
		if (status < 0){
			fail++;
			time = xTaskGetTickCount() - last_time;
			//printf("Send failed\r\n");
			//printf("s%it%i\r\n", i, 0);
		}
		else {
			time = xTaskGetTickCount() - last_time;
			avg_time += time;
			
			//printf("s%it%i\r\n", i, time);
			printf("s%it%" PRIu64 "\r\n", i, (long long unsigned int) xTaskGetTickCount());
		}
		//if (nuts_modules[this_module].csp_address == 3) vTaskDelay(100);
		
	}
	printf("total time= %" PRIu64 " sec\r\n", (uint64_t)((xTaskGetTickCount() - start)/1000));
	printf("avg_time of stream: %f, fails: %i\r\n", (float)((float)avg_time / (float)(tries - fail)), fail);
}


/*Listen example task*/
void example_listen_task(void *ptr) {
	uint8_t listen_port = *(uint8_t*)ptr;
	
	printf("\rListener task started!\r\n");
	
	if (nrp_bind_port(listen_port) != SUCCESS){
		vTaskDelete( NULL );
		while(true);
	}
	
	nrp_recv_packet_t * packet;
	
	while(true) {
		packet = nrp_read(listen_port, 1000);
		if (packet == NULL) {
			vTaskDelay(10);
			continue;
		}
		
		char ch;
		Bool match = true;
		for (int i = 0; i < strlen(huge_packet); i++){
			ch = *(huge_packet + (i*sizeof(char)));
			if ((char)packet->data[i] != ch) {
				match = false;
				break;
			}
		}
		
		
		if (!match) printf("\r\n[%i:%i -> %i]: huge_packet received incorrectly (500 bytes)\r\n", packet->src.csp_address, packet->src.port, packet->dest.port);
		//else printf("\r\n[%i:%i -> %i]: %s\r\n", packet->src.csp_address, packet->src.port, packet->dest.port, packet->data);
		//
		//uint64_t transmissions = strtoull(packet->data, (char **)NULL, 10);
		//char *buf;
		//sprintf(buf, "%" PRIu64 "", ++transmissions);
		//
		//module_socket_t dest;
		//
		
		//switch (nuts_modules[this_module].csp_address){
			//case 1:
			//printf("r[%" PRIu32 "]:t%" PRIu64 "\r\n", ++trans, (uint64_t)xTaskGetTickCount());
			//dest.csp_address = 2;
			//dest.port = 10;
			//nrp_send(&dest, &packet->dest, buf, sizeof(buf), CSP_PRIO_NORM);
			//break;
			//
			//case 2:
			//dest.csp_address = 3;
			//dest.port = 10;
			//nrp_send(&dest, &packet->dest, buf, sizeof(buf), CSP_PRIO_NORM);
			//break;
			//
			//case 3:
			//dest.csp_address = 1;
			//dest.port = 10;
			//nrp_send(&dest, &packet->dest, buf, sizeof(buf), CSP_PRIO_NORM);
			//break;
		//}
		
		
		free_nrp_packet(packet);
		//printf("Heap: %i\r\n", xPortGetFreeHeapSize());
		
	}
	
}

void example_printout_listen_task(void *ptr) {
	uint8_t listen_port = *(uint8_t*)ptr;
	
	uint16_t from_rad = 0;
	uint16_t from_cam = 0;
	
	uint64_t start_diff = 0;
	printf("\rListener task started!\r\n");
	
	if (nrp_bind_port(listen_port) != SUCCESS){
		vTaskDelete( NULL );
		while(true);
	}
	
	nrp_recv_packet_t * packet;
	
	while(true) {
		packet = nrp_read(listen_port, 1000);
		if (packet == NULL) {
			vTaskDelay(10);
			continue;
		}
		
		char ch;
		Bool match = true;
		if (packet->src.csp_address == 2){
			for (int i = 0; i < strlen(huge_packet); i++){
				ch = *(huge_packet + (i*sizeof(char)));
				if ((char)packet->data[i] != ch) {
					match = false;
					break;
				}
			}
		}
		else {
			for (int i = 0; i < strlen("hola"); i++){
				ch = *("hola" + (i*sizeof(char)));
				if ((char)packet->data[i] != ch) {
					match = false;
					break;
				}
			}
		}
		
		//if (match) printf("\r\n[%i:%i -> %i]: huge_packet received (500 bytes)\r\n", packet->src.csp_address, packet->src.port, packet->dest.port);
		//else printf("\r\n[%i:%i -> %i]: %s\r\n", packet->src.csp_address, packet->src.port, packet->dest.port, packet->data);
		
		if (start_diff == 0) start_diff = xTaskGetTickCount();
		uint64_t time = xTaskGetTickCount() - (uint64_t)start_diff;
		
		if (match){
			
			//printf("%i:%i[%" PRIu64 "]\r\n", packet->src.csp_address, packet->src.csp_address == 2 ? ++from_rad : ++from_cam, time);
			switch (packet->src.csp_address){
				case 2:
					//diff = (uint16_t)(xTaskGetTickCount() - from_rad);
					//from_rad = xTaskGetTickCount();
					//printf("2:%i[%lu]\r\n", ++from_rad, (unsigned long)time);
					printf("2:%i[%" PRIu64 "]\r\n", ++from_rad, time);
				break;
				
				case 3:
					//diff = xTaskGetTickCount() - from_cam;
					//from_cam = xTaskGetTickCount();
					printf("3:%i[%" PRIu64 "]\r\n", ++from_cam, time);
					
				break;
			}
		}
		else printf("%i:-1 No match\r\n",packet->src.csp_address);
		
		free_nrp_packet(packet);
		vTaskDelay(1);
	}
	
}

void example_send_nrp_message(portCHAR * str){
	char *rest = str;
	
	char *message = strsep(&rest, "@");
	uint8_t address = atoi(strsep(&rest, ":"));
	uint8_t port = atoi(rest);
	//printf("\rAddress: %i:%i, Message: %s\r\n", address, port, message);
	
	
	nuts_module_t to_module = nuts_modules[address - 1];
	
	module_socket_t to_socket = {to_module.csp_address, port};
	module_socket_t from_socket = {nuts_modules[this_module].csp_address, 10};
	
	if (strcmp(message, "ping") == 0) message = ping_packet;
	else if (strcmp(message, "big") == 0) message = big_packet;
	else if (strcmp(message, "huge") == 0) message = (char*)huge_packet;
	
	
	int8_t status = nrp_send(&to_socket, &from_socket, (uint8_t*) message, strlen(message), CSP_PRIO_NORM);
	if (status != SUCCESS){
		NRP_PRINT_ERROR("Not able to send message");
	}	
}


void example_send_unrp_message(portCHAR * str){
	char *rest = str;
	
	char *message = strsep(&rest, "@");
	uint8_t address = atoi(strsep(&rest, ":"));
	uint8_t port = atoi(rest);
	//printf("\rAddress: %i:%i, Message: %s\r\n", address, port, message);
	
	
	nuts_module_t to_module = nuts_modules[address - 1];
	
	module_socket_t to_socket = {to_module.csp_address, port};
	module_socket_t from_socket = {nuts_modules[this_module].csp_address, 20};
	
	if (strcmp(message, "ping") == 0) message = ping_packet;
	else if (strcmp(message, "big") == 0) message = big_packet;
	else if (strcmp(message, "huge") == 0) message = (char*)huge_packet;
	
	
	int8_t status = nrp_send_unreliable_datagram(&to_socket, &from_socket, (uint8_t*) message, strlen(message), CSP_PRIO_NORM);
	if (status != SUCCESS){
		NRP_PRINT_ERROR("Not able to send message");
	}
	printf("Sending unrp msg\r\n");
}