/*
 * csp_util.c
 *
 * Created: 09-Nov-12 1:47:50 PM
 *  Author: andrasg
 */ 
#include "csp_util.h"
#include "conf_csp.h"
#include "FreeRTOSConfig.h"
#include "task.h"
#include "csplib/include/csp/csp.h"
#include "csplib/include/csp/interfaces/csp_if_i2c.h"
//#include "io/uart/uart_write.h"
#include <string.h>
#include <stdio.h>

/************************************************************************/
/* Prototypes                                                           */
/************************************************************************/
/**
 * Private function that is called to "clean" a csp packet.
 * \param pointer to the packet to "clean"
 * \param size of the data that sould be available
 * \return a "clean" csp_packet, or NULL
 */
void * csp_clean_packet(csp_packet_t * packet, uint8_t size);

/************************************************************************/
/* Variables                                                            */
/************************************************************************/
//! Connecting some addresses and ports, only ports are used in the testing.
const addr_port_t modules[4] = {
		{.addr = 0x1, .port = OBC_PORT},
		{.addr = 0x3, .port = RAD_PORT},
		{.addr = 0x5, .port = ACDS_PORT},
		{.addr = 0x7, .port = OTHER_PORT}
};

/************************************************************************/
/* Functions                                                            */
/************************************************************************/

int csp_usage_init(int buffer_n, uint8_t my_address, unsigned int routing_stacksz){
	int err;
	//Init CSP
	//CSP guide sets the buffer inits to substantially larger than here, 300, 324 to be exact
	if (err = csp_buffer_init(buffer_n, CSP_MAX_PACKET_SIZE) != CSP_ERR_NONE){
		printf("\rUnable to initialize CSP buffer: %i\r\n", err);
		return -1;
	}		
	
	//A safety on the csp address, since the addressspace is only 4 bits.	
	uint8_t csp_address = my_address & 0x0F;
	
	err = csp_init(csp_address);
	if (err != CSP_ERR_NONE){
		printf("\rUnable to initialize CSP: %i\r\n", err);
		return -1;
	}		
		
	//Init I2C
	/*Something was wrong with the return value of csp_i2c_init, therefore >= 0 instead of != CSP_ERR_NONE*/
	err = csp_i2c_init(my_address, 0, I2C_BAUDRATE);
	if (err >= 0){
		printf("\r\nUnable to initialize i2c, error: %i\r\n", err);
		return -1;
	}		
	
		
	//Init routing of packets
	err = csp_route_set(CSP_DEFAULT_ROUTE, &csp_if_i2c, CSP_NODE_MAC);
	if (err != CSP_ERR_NONE){
		printf("Unable to setup routing: %i\r\n", err);
		return -1;
	}		
	
	err = csp_route_start_task(routing_stacksz, configTSK_ROUTINGTASK_PRIORITY);
	if (err != CSP_ERR_NONE){
		printf("Unable to start routing task: %i\r\n", err);
		return -1;
	}
	return 0;
}


void * csp_clean_packet(csp_packet_t * packet, uint8_t size){
	if (packet == NULL)
		return NULL;
	//clean_id
	packet->id.ext = 0;
	//clean_length
	packet->length = 0;
	//clean_padding
	for (int i = 0; i < CSP_PADDING_BYTES; i++){
		packet->padding[i] = '\0';
	}

	return packet;
}

void * csp_buffer_get_isr_clean(uint8_t size){
	csp_packet_t *packet= csp_buffer_get_isr(size);
	return csp_clean_packet(packet, size);
}

void * csp_buffer_get_clean(uint8_t size){
	csp_packet_t *packet= csp_buffer_get(size);
	return csp_clean_packet(packet, size);
}

int csp_listen_socket_all(csp_socket_t **out){
	//create the socket
	*out = csp_socket(CSP_SO_NONE);
	if (*out == NULL)
	return -1;
	//Bind socket to all ports
	if (csp_bind(*out, CSP_ANY) != CSP_ERR_NONE)
	return -2;
	//Activate listening for socket
	if (csp_listen(*out, BACKLOG_CON_QUEUE) != CSP_ERR_NONE)
	return -3;
	return CSP_ERR_NONE;
}
