/*
 * csp_util.h
 *
 *  Author: andrasg
 */ 


#ifndef CSP_UTIL_H_
#define CSP_UTIL_H_

#include "FreeRTOS.h"
#include "csplib/include/csp/csp.h"
#include "conf_csp.h"

/* Must be noted this is just an experiment for the testing, not something to be 
 * used in the later use of the implementation. And a module can have multiple
 * ports.
 */
//! The port the OBC will listen to
#define OBC_PORT MASTER_PORT
//! The port the Radio will listen to
#define RAD_PORT SLAVE_PORT
//! The port the ACDS will listen to
#define ACDS_PORT 0x15
//! Some other port
#define OTHER_PORT 0x17
//! Struct with an i2c address and a port, which can dictate the address for a task.
typedef struct {
	uint8_t addr;
	uint8_t port;
}addr_port_t;

//! A enum to make picking out addresses easier
typedef enum cubesat_module{
	OBC = 0,
	RAD = 1,
	CAM = 2,
	ADCS = 3,
}cubesat_module;
//! A list of the modules and their address and port.
extern const addr_port_t modules[4];
/**
 * A function that does all the essential initialisation to use csp and 
 * i2c.
 * \param buffer_n number of elements in the csp buffer
 * \param my_address the address for both the i2c module and csp module
 * \routing_stacksz stack size of the routing task
 * \return 0 if successful, -1 if failed.
 */
int csp_usage_init(int buffer_n, uint8_t my_address, unsigned int routing_stacksz);

/**
 * Function to get a socket for a enum cubesat_module.
 * Will create a socket, bind it and start listening to it.
 * Port is based on the port in the modules array.
 * \param module an enum that decides what position in the modules array will be used for the creation
 * \return a csp_socket_t, or NULL.
 */
void * csp_listen_socket_module(cubesat_module module);

/**
 * Function to get a socket that listens to all the ports.
 * \param out a csp_socket_t out variable
 * \return -1 if failed socket cration, -2 if failed bind, -3 if listen failed
 */
int csp_listen_socket_all(csp_socket_t **out);

/**
 * Function that gets a new packet from the buffer, and "cleans" it, 
 * by removing most of the persistant data that could make trouble.
 * this is mostly a relic from early code by inputting '\0' at the 
 * "correct" place to make string compare work on the data that has
 * been sent.
 * \param size of the data that sould be available
 * \return a "clean" csp_packet, or NULL
 */
void * csp_buffer_get_clean(uint8_t size);

/**
 * Same as csp_buffer_get_clean(uint8_t size), but is usable from
 * within an ISR.
 * \param size of the data that sould be available
 * \return a "clean" csp_packet, or NULL
 */
void * csp_buffer_get_isr_clean(uint8_t size);

#endif /* CSP_UTIL_H_ */