/*
 * i2c_csp_glue.c
 *
 *  Author: andrasg
 */ 
#include "i2c_csp_glue.h"
#include <stdio.h>

#include "task.h"
//#include "io/uart/uart_write.h"

#include "conf_csp.h"

#include "csplib/csp_util.h"	
#include "i2c/i2c_handler.h"
#include "csplib/CSP_testing/csp_testing.h"
#include "i2c/i2c_master_interupt.h"

/************************************************************************/
/* Prototypes                                                           */
/************************************************************************/
//! Helper function to get names from status values in the status_code.h file.
const char * name_of_status(status_code_t status);
int i2c_init_glueglue( unsigned int addr, callback_csp_lib_rx csp_rx );


/************************************************************************/
/* Variables                                                            */
/************************************************************************/
//! Pointer to the frame buffer we are using as a receiving buffer
static volatile i2c_frame_t * recieve = NULL;

//! The callback function received from csp. so for each packet received, we should call this.
static volatile callback_csp_lib_rx callback_csp_rx;

//! Variable with the size of the csp overhead that's sent across the bus.
static volatile uint8_t overhead_size = 4;

int rx_success = 0;
int rx_fail = 0;
int tx_success = 0;
int tx_fail = 0;
int rx_total = 0;

/************************************************************************/
/* Functions called from CSP framework                                  */
/************************************************************************/
int i2c_init(int handle, int master, unsigned int addr, int speed,int unknown1, int unknown2, callback_csp_lib_rx csp_rx){
	
	int err = E_NO_ERR;
	
	callback_csp_rx = csp_rx;
	
	/*Reset pointer to receiving buffer*/
	recieve = NULL;
	
	if ((err = i2c_enable_hardware()) != E_NO_ERR) return err;
	if ((err = i2c_enable_incomming(addr)) != E_NO_ERR) return err;
	if ((err = i2c_enable_outgoing()) != E_NO_ERR) return err;
	
	return E_NO_ERR;
}


int i2c_send(int handle, i2c_frame_t * frame, unsigned int timeout)
{	
	status_code_t status = i2c_send_csp_packet(frame, timeout);
	if (STATUS_OK == status){
	//csp expects frames that's been sent to be freed
		csp_buffer_free(frame);
		tx_success ++;
	}else {
		//printf("Send : [%i : %s]\n\r", status, name_of_status(status));
		tx_fail ++;
	}	
	return status;
}

void clear_counters(void){
	rx_success = 0;
	rx_fail = 0;
	tx_success = 0;
	tx_fail = 0;
}

/************************************************************************/
/* Functions that get called from handler                               */
/************************************************************************/
status_code_t i2c_create_recieve_buffer(void)
{
	if (recieve == NULL){
		recieve = csp_buffer_get_isr_clean(CSP_MAX_DATA_SIZE - sizeof(csp_id_t)); //csp max data - 4
	}
	return recieve != NULL ? STATUS_OK : ERR_PACKET_ALLOCATION;
}

status_code_t i2c_receive(uint8_t recv_byte)
{	
	status_code_t ret_val = ERR_PACKET_ALLOCATION;
	
	//See if the byte recv is a header or a data byte
	if (recieve != NULL && recieve->len_rx < overhead_size){
	// Is a header byte
		recieve->id.ext |= ((uint32_t)recv_byte) << ((overhead_size - (recieve->len_rx + 1))*8);
		recieve->len_rx ++;
		ret_val = STATUS_OK;
	}else if (recieve != NULL){
	//Recv a data byte
		if (recieve->len < CSP_MAX_DATA_SIZE){
			recieve->data[recieve->len] = recv_byte;
			recieve->len_rx ++;
		}
		recieve->len ++;
		ret_val = STATUS_OK;
	}
	return ret_val;
}

portBASE_TYPE i2c_clear_receive(status_code_t stop_type){
	
	portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;	
	if (STATUS_OK == stop_type && recieve != NULL){
	//Transfer is successful
		//rx_total++;
		//if(!(rx_total%50))
		//	printf("I2C packet count: %i\r\n", rx_total);
		//printf("I2C packet length: %i\r\n", recieve->len);
		recieve->length = recieve->len;
		callback_csp_rx((i2c_frame_t*)recieve, (void*)&xHigherPriorityTaskWoken);
		recieve = NULL;
		rx_success++;
	}else{	
	//Failed the transfer, and should clear the buffer
		if (recieve != NULL){
			csp_buffer_free_isr((void*)recieve);
			recieve = NULL;
		}		
		rx_fail ++;
	}
	return xHigherPriorityTaskWoken;
}

/**
 * \brief Helper function to get names from status values in the status_code.h file.
 * \param Status status code of the status you want the name of
 * \return Name of the status code.
 */
const char * name_of_status(status_code_t status){
	switch (status) {
		case ERR_IO_ERROR :
			return "ERR_IO_ERROR";
		case ERR_FLUSHED :
			return "ERR_FLUSHED";
		case ERR_TIMEOUT :
			return "ERR_TIMEOUT";
		case ERR_BAD_DATA :
			return "ERR_BAD_DATA";
		case ERR_PROTOCOL :
			return "ERR_PROTOCOL";
		case ERR_UNSUPPORTED_DEV :
			return "ERR_UNSUPPORTED_DEV";
		case ERR_NO_MEMORY:
			return "ERR_NO_MEMORY";
		case ERR_INVALID_ARG:
			return "ERR_INVALID_ARG";
		case ERR_BAD_ADDRESS :
			return "ERR_BAD_ADDRESS";
		case ERR_BUSY:
			return "ERR_BUSY";
		case ERR_BAD_FORMAT:
			return "ERR_BAD_FORMAT";
		case ERR_PACKET_ALLOCATION:
			return "ERR_PACKET_ALLOCATION";
		case ERR_SEMAPHORE_FAIL:
			return "ERR_SEMAPHORE_FAIL";
		default:
			return "STATUS_OK";
	}	
}	