/*
 * i2c_slave_interupt.h
 *
 *  Author: andrasg
 */ 
#ifndef I2C_SLAVE_INTERUPT_H_
#define I2C_SLAVE_INTERUPT_H_

#include <avr32/io.h>
#include <stdint.h>
#include "compiler.h"
#include "status_codes.h"
#include "conf_twis.h"
#include "FreeRTOS.h"

/************************************************************************/
/* Structs                                                              */
/************************************************************************/
//! Struct used the initialization of the i2c slave functionality, contains speed and address. 
typedef struct
{
	//! The PBA clock frequency.
	uint32_t pba_hz;
	//! The baudrate of the TWI bus.
	uint32_t speed;
	//! The desired address.
	uint8_t chip;
}
twis_options_t;

/************************************************************************/
/* Variables                                                            */
/************************************************************************/
//! Pointer to the twis module which is used throughout i2c_slave_interupt.
extern volatile avr32_twis_t *twis_inst;

/************************************************************************/
/* Functions for initial settings in I2C slave                          */
/************************************************************************/
/**
 * \brief Function for registering the TWIS IRQ
 * 
 * \param twis Memory area of the twis module, cast into a struct that makes it easier to explore it.
 * \param irq IRQ for the intertupt to register.
 */
void twis_register_interupt(volatile avr32_twis_t *twis, unsigned int irq);

/**
 * \brief Function for default setup of TWIS based of a option struct.
 * 
 * \param twis Memory area of the twis module, cast into a struct that makes it easier to explore it.
 * \param opt Pointer to a struct containing basic information about the slave to be enabled
 */
void twis_enable_settings(volatile avr32_twis_t *twis, const twis_options_t *opt);

#endif /* I2C_SLAVE_INTERUPT_H_ */