%**************************************************************************
% File currentScaling.m
%
% [m_B, W] = currentScaling(P,m_B)
%
% Scales the magnetic moment m_B so that it does not use a higher current
% on each coil than what is available for the real system
%
% Written by Zdenko Tudor, 2011. Edited by Marius Westgaard, 2014
%**************************************************************************

function [m_B, W] = currentScaling(P,m_B)
% Load coil parameters from parameter struct
Nx = P.Nx; Ny = P.Ny; Nz = P.Nz;
Ax = P.Ax; Ay = P.Ay; Az = P.Az;
ix_max = P.ix_max; iy_max = P.iy_max; iz_max = P.iz_max;
V = P.V;

% Current scaling
% Creates ratio variable containing a coil's: max current/wanted current.
% The lowest value in ratio variable, if below 1 is the highest current
% violation, thus all the currents should be scaled by this factor.
ix = m_B(1)/(Nx*Ax); ratio(1)=ix_max/abs(ix);
iy = m_B(2)/(Ny*Ay); ratio(2)=iy_max/abs(iy);
iz = m_B(3)/(Nz*Az); ratio(3)=iz_max/abs(iz);

if min(ratio)<1
    m_B = m_B*min(ratio);
end

% Scale the lower range of the current
if (abs(m_B(1)) < 0.0000082644)
    m_B(1) = 0;
end
if (abs(m_B(2)) < 0.0000082644)
    m_B(2) = 0;
end
if (abs(m_B(3)) < 0.0000082644)
    m_B(3) = 0;
end


% Power consumption (Joules)
Wx = abs(V*m_B(1)/(Nx*Ax));
Wy = abs(V*m_B(2)/(Ny*Ay));
Wz = abs(V*m_B(3)/(Nz*Az));
W = Wx+Wy+Wz;
end
