%**************************************************************************
% File detumbling_controller.m
%
% [tau_m, m_B, W] = detumbling_controller(P, B_B)
%
% Calculates the torque, magetic moment and power usage generated
% by the controller. To be used with nonlinearSatellite.m
%
% Written by Marius Westgaard, 2015
%**************************************************************************

function [tau_m, m_B, W, B_dot_estimate] = detumbling_controller(P, B_B, B_B_prev, B_dot_estimate_prev)

% Estimator constants
omega_c = 0.7;
T = 1;
K = 1;
a = exp(-(omega_c*T));
b = K*omega_c;

B_dot_estimate = a*B_dot_estimate_prev + b*(B_B - B_B_prev);

% Controller gain
k = 0.0001;

% Moment set up by coils before scaling (saturating current)
m_B = -k*(1/(norm(B_B,2)^2))*B_dot_estimate;

% Moment set up by coils after scaling
[m_B, W] = currentScaling(P,m_B);

% Torque set up by coils
tau_m = cross(m_B,B_B);

end

