%**************************************************************************
% File IGRF11_load_data.m
%
% Section 1:
% Create Matlab variables g_data and h_data containing g and h
% coefficients for the IGRF model from an excel file found online. The
% coefficients stored in matlab variables are compensated with the secular
% variation.
%
% Section 2:
% Generate C header file with the IGRF coefficients to be used on a
% microcontroller.
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

[data,txt] = xlsread('IGRF11_data_file.xlsx');

O = 1;
for i=1:length(data)
    col1 = data(i,1);
    col2 = data(i,2);
    col3 = data(i,3);
    col4 = data(i,4);
    if strcmp(txt(i),'g')
        g_data(O+col1,O+col2) = col3;
        g_dataSV(O+col1,O+col2) = col4;
    elseif strcmp(txt(i),'h')
        h_data(O+col1,O+col2) = col3;
        h_dataSV(O+col1,O+col2) = col4;
    end
end
g_data = g_data+(1+53/365)*g_dataSV;
h_data = h_data+(1+53/365)*h_dataSV;

save IGRF11_data g_data h_data

%% Generate C header file for IGRF coefficients
filename = 'IGRFcoeffs.h';

load('IGRF11_data');
fid = fopen(filename,'w');

% Start the header file and include some of the constants
fprintf(fid, 'static const double h_data[14*14] = {+\n');

L = size(h_data,1);
O = 1;
for i=0:L-1
    for j=0:L-1
        fprintf(fid,' %.3f,', h_data(i+O,j+O));
        if mod(i*L+j+1,10) == 0
            fprintf(fid,'+\n');
        end
    end
end
fseek(fid,-1,'eof'); fprintf(fid,''); % Remove last comma character
fprintf(fid,'};\n\n');

fprintf(fid, 'static const double g_data[14*14]={+\n');
for i=0:L-1
    for j=0:L-1
        fprintf(fid,' %.3f,', g_data(i+O,j+O));
        if mod(i*L+j+1,10) == 0
            fprintf(fid,'+\n');
        end
    end
end
fseek(fid,-1,'eof'); fprintf(fid,''); % Remove last comma character
fprintf(fid,'};\n\n');
fprintf(fid, '\n#endif /*_IGRFcoeffs_*/\n');

fclose(fid);
