%**************************************************************************
% File outFcn.m
%
% status = outFcn(t,~,flag)
%
% Is run when ODE45 successfully completes a step.
% This function is used implicitly by ODE45, when ODE settings are
% properly set.
%
% Example:
% options = odeset('OutputFcn',@outFcn,'Refine',1);
% Refine sets how many steps should be created for each successfull step.
% With this set to 1 length of all vectors created by the OutputFcn will be
% the same as the length of ODE timevector TOUT.
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

function status = outFcn(t,~,flag)
persistent ite;
global tmpVAR VAR;
switch flag
    case 'init'
        %initialize arrays etc.
        ite = 1;
        VAR.moment = zeros(10000,3);
        VAR.torque = zeros(10000,3);
        VAR.J = zeros(10000,1);
        VAR.t = zeros(10000,1);
        
        status = 0;
    case 'done'
        VAR.moment(ite+1:end,:) = [];
        VAR.torque(ite+1:end,:) = [];
        VAR.J(ite+1:end,:) = [];
        VAR.t(ite+1:end,:) = [];
        
        status = 0;
    otherwise
        ite = ite+1;
        VAR.moment(ite,:)=tmpVAR.moment';
        VAR.torque(ite,:)=tmpVAR.torque';
        VAR.W(ite,:)=tmpVAR.W;
        VAR.t(ite,1)=t(end);
        
        status = 0;
end
end