%**************************************************************************
% File detumbling_controller.m
%
% [tau_m, m_B, W] = tumbling_controller(P, w_B_IB, B_B, q)
%
% Calculates the torque, magetic moment and power usage generated
% by the controller. To be used with nonlinearSatellite.m
%
% Written by Marius Westgaard, 2015
%**************************************************************************

function [tau_m, m_B, W] = tumbling_controller(P, w_B_IB, B_B, q)
% Epsilon crossmatrix, skew-symmetric
S_eps = [ 0, -q(4), q(3);   
          q(4), 0, -q(2);
         -q(3), q(2), 0];

% Rotation matrices
R_O_B = eye(3) + 2*q(1)*S_eps + 2*S_eps^2;
R_B_O = R_O_B';

% Angular velocities of frames relative each other
w_o = P.w_o;
w_O_IO = [0;-w_o;0];
w_B_OB = w_B_IB-R_B_O*w_O_IO;

% Controller gains
h = 1;

% Moment set up by coils before scaling (saturating current)
m_B = h*(1/norm(B_B,2)^2)*cross(B_B,w_B_OB);

% Moment set up by coils after scaling
[m_B, W] = currentScaling(P,m_B);

% Torque set up by coils
tau_m = cross(m_B,B_B);
end

